package fr.free.supertos.anniversaire;

import java.io.InputStream;
import java.lang.reflect.Field;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.graphics.drawable.Drawable;
import android.net.Uri;
import android.os.Build;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.text.Editable;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.DatePicker;
import android.widget.EditText;
import android.widget.ImageButton;
import android.widget.TextView;
import fr.free.supertos.anniversaire.bean.BeanAnniversaires;
import fr.free.supertos.anniversaire.constantes.ConstantesAnniversaires;
import fr.free.supertos.anniversaire.dao.DaoAnniversaires;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.database.classes.ContactAnniv;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;
import fr.free.supertos.anniversaire.service.ServiceAutoRun;

/**
 * Classe permettant la creation d'un vnement
 * 
 * @author supertos
 *
 */
public class CreationEventActivity extends Activity implements OnClickListener {
	
	private final static String CLASSE = CreationEventActivity.class.getName();
	
	BeanAnniversaires bAnniversaire;
	
	private EditText editTextPrenom;
	private EditText editTextNom;
	private DatePicker datePicker;
	private CheckBox checkBoxAnnee;
	private CheckBox checkBoxAutreEven;
	private ImageButton buttonAssocier;
	
	private Integer id; // Id dans la base anniversaire
	private long idContact;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_creation);
		
		// Rcuprer l'id ventuellement transmis par intention
		Intent intent = getIntent();
		int idExtra=intent.getIntExtra("id", -1); // Si rien retourner -1
		
		bAnniversaire=new BeanAnniversaires();
		
		if (idExtra==-1)
		{
			id=null;
		}
		else
		{
			id=idExtra;
        	SqLiteAnniversaire sqLiteAnniv = new SqLiteAnniversaire();
    		SQLiteDatabase idSqLite=sqLiteAnniv.openDB(this);
    		
    		DaoAnniversaires daoAnniversaires=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
    		
    		bAnniversaire = daoAnniversaires.lireAnniversaire(id);
    		
    		sqLiteAnniv.closeDB (idSqLite);
		}
		
		buttonAssocier=(ImageButton)findViewById(R.id.buttonAssocier);
		buttonAssocier.setOnClickListener((OnClickListener) this);
		
		Button buttonValider=(Button)findViewById(R.id.buttonValider);
		buttonValider.setOnClickListener((OnClickListener) this);
		
		Button buttonAnnuler=(Button)findViewById(R.id.buttonAnnuler);
		buttonAnnuler.setOnClickListener((OnClickListener) this);
		
		datePicker =(DatePicker)findViewById(R.id.datePickerAnniversaire);
		if (Build.VERSION.SDK_INT>=Build.VERSION_CODES.HONEYCOMB)
		{
			// Dsactivation de l'affichage du calendrier  la place de l'anne
			datePicker.setCalendarViewShown(false);
		}
		
		editTextPrenom=(EditText)findViewById(R.id.editTextPrenom);
		editTextNom=(EditText)findViewById(R.id.editTextNom);
		
		checkBoxAnnee=(CheckBox) findViewById(R.id.checkBoxYearUnknow);
		checkBoxAnnee.setOnClickListener((OnClickListener) this);
		
		checkBoxAutreEven=(CheckBox) findViewById(R.id.checkBoxAutreEven);
		checkBoxAutreEven.setOnClickListener((OnClickListener) this);
		
		editTextPrenom.setEnabled(true);
		editTextNom.setEnabled(true);
		
		// Restaurer les donnes prcdemments saisies. cf. onSaveInstanceState
		restorePrecedenteSaisie (savedInstanceState);
	}

	public void onPause()
	{
		super.onPause();
		
		// On en profite pour stocker les informations dans le bean
	    Editable prenom =  editTextPrenom.getText();
	    bAnniversaire.setPrenom(prenom.toString());
	    
	    Editable nom = editTextNom.getText();
	    bAnniversaire.setNom(nom.toString());
	    
	    bAnniversaire.setIdContact(idContact);
	    bAnniversaire.setAnnee(datePicker.getYear());
	    bAnniversaire.setMois(datePicker.getMonth()+1);
	    bAnniversaire.setJour(datePicker.getDayOfMonth());
   
		bAnniversaire.setAnneeConnue(!checkBoxAnnee.isChecked());
		bAnniversaire.setAutreEven(checkBoxAutreEven.isChecked());
	}
		
	public void onResume()
	{
		super.onResume();

		if (LogAnniv.isDebug())
		{
			LogAnniv.d(CLASSE, "datePicker"+datePicker+", date="+bAnniversaire.getAnnee()+"/"+ bAnniversaire.getMois()+"/"+bAnniversaire.getJour()+ ", id contact="+idContact);
		}

		idContact=bAnniversaire.getIdContact();

		modeAnnivAutre(bAnniversaire.getAutreEven());
		editTextNom.setText (bAnniversaire.getNom());
		checkBoxAnnee.setChecked(bAnniversaire.getAnneeConnue() ? false:true);
		checkBoxAutreEven.setChecked(bAnniversaire.getAutreEven());
		
		datePicker.init(bAnniversaire.getAnnee(), bAnniversaire.getMois()-1, bAnniversaire.getJour(), null);
		
		if (bAnniversaire.getIntAnneeConnue()==ConstantesAnniversaires.ANNEECONNUE)
		{
			setVisibiltyYear(true);
		}
		else
		{
			setVisibiltyYear(false);
		}
		
		if (idContact!=-1)
		{
			ContactAnniv contactAnniv=new ContactAnniv(idContact, getContentResolver());
			afficherPhotoContactBouton(contactAnniv);
		}
	}

	private void restorePrecedenteSaisie(Bundle savedInstanceState) {
		// Restaurer les paramtres saisies suite  rotation de l'cran, mmoire insuffisante, ...
		if (LogAnniv.isDebug())
		{
			LogAnniv.d(CLASSE, "Restore !"+savedInstanceState);
		}
		
		if (savedInstanceState != null)
		{
			bAnniversaire.setPrenom(savedInstanceState.getString("prenom"));
			bAnniversaire.setNom(savedInstanceState.getString("nom"));
			
			bAnniversaire.setAnnee(savedInstanceState.getInt("annee"));
			bAnniversaire.setMois(savedInstanceState.getInt("mois"));
			bAnniversaire.setJour(savedInstanceState.getInt("jour"));
			
			if (Build.VERSION.SDK_INT>=Build.VERSION_CODES.HONEYCOMB)
			{
				// Dsactivation de l'affichage du calendrier  la place de l'anne
				datePicker.setCalendarViewShown(false);
			}
			
			if (savedInstanceState.getBoolean("anneeInconnue"))
			{
				bAnniversaire.setAnneeConnue(false);
			}
			else
			{
				bAnniversaire.setAnneeConnue(true);
			}
			
			bAnniversaire.setIdContact (savedInstanceState.getLong("idContact"));
			
			bAnniversaire.setAutreEven(savedInstanceState.getBoolean("autreEvent"));
			modeAnnivAutre(savedInstanceState.getBoolean("autreEvent"));
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle saveInstanceState) {

		// Sauver les paramtres saisies si rotation de l'cran, mmoire insuffisante, ...
		// Nous pourrons ainsi les restaurer ensuites !
		
		if (LogAnniv.isDebug())
		{
			LogAnniv.d(CLASSE, "Save !");
		}

	    
	    saveInstanceState.putString("prenom", bAnniversaire.getPrenom());
	    saveInstanceState.putString("nom", bAnniversaire.getNom());   
	    saveInstanceState.putLong("idContact", bAnniversaire.getIdContact());
	    saveInstanceState.putInt ("annee", bAnniversaire.getAnnee());
	    saveInstanceState.putInt ("mois", bAnniversaire.getMois());
	    saveInstanceState.putInt ("jour", bAnniversaire.getJour());
   
		saveInstanceState.putBoolean("anneeInconnue", !bAnniversaire.getAnneeConnue());
		saveInstanceState.putBoolean("autreEvent", bAnniversaire.getAutreEven());
		
	    super.onSaveInstanceState(saveInstanceState);

	}
	
	// Suivant qu'il s'agit d'un anniversaire d'une personne ou d'un autres vnements,
	// certains champs seront visibles ou non
	private void modeAnnivAutre(Boolean mode) {
		TextView textViewNom=(TextView)findViewById(R.id.textViewNom);
		TextView textViewPrenom=(TextView)findViewById(R.id.textViewPrenom);
		
		if (mode)
		{
			editTextPrenom.setVisibility(View.INVISIBLE);
			textViewPrenom.setVisibility(View.INVISIBLE);
			textViewNom.setText(R.string.texteIntitule);
		}
		else
		{
			editTextPrenom.setText(bAnniversaire.getPrenom());
			textViewNom.setText(R.string.texteNom);
			editTextPrenom.setVisibility(View.VISIBLE);
			textViewPrenom.setVisibility(View.VISIBLE);
		}
	}

	// Gestion du click sur les boutons/checkbox
	public void onClick(View view){
    	switch (view.getId())
    	{	    	
	    	case R.id.buttonAssocier:
	    		Intent intent = new Intent(Intent.ACTION_PICK);
	            intent.setType(ContactsContract.Contacts.CONTENT_TYPE);
	            startActivityForResult(intent, 1); 
	    	break;

	    	case R.id.buttonValider:
	    		String prenomSaisi=editTextPrenom.getText().toString();
	    		String nomSaisi=editTextNom.getText().toString();
		        if (prenomSaisi.isEmpty() && nomSaisi.isEmpty())
		        {
		        	AlertDialog.Builder mesAlert = new AlertDialog.Builder(this);
		       
		        	if (bAnniversaire.getAutreEven())
		        	{
		        		mesAlert.setTitle(R.string.alert_title_intitule_non_renseigne);
		        		mesAlert.setMessage(R.string.alert_mes_intitule_non_renseignee);		        		
		        	}
		        	else
		        	{
		        		mesAlert.setTitle(R.string.alert_title_indetite_non_renseignee);
		        		mesAlert.setMessage(R.string.alert_mes_indetite_non_renseignee);
		        	}
		            // Dclaration des boutons  utiliser
		            mesAlert.setNeutralButton(android.R.string.ok, null);		            
		            // Affichage du message
		            mesAlert.show();
		        	
		        }
		        else
		        {
		        	SqLiteAnniversaire sqLiteAnniv = new SqLiteAnniversaire();
		    		SQLiteDatabase idSqLite=sqLiteAnniv.openDB(this);
		    		
		    		DaoAnniversaires daoAnniversaires=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
		    		BeanAnniversaires bAnniversaire=new BeanAnniversaires();
		    		bAnniversaire.setId(id);
		    		bAnniversaire.setIdContact(idContact);
		    		bAnniversaire.setNom(nomSaisi);
		    		bAnniversaire.setPrenom(prenomSaisi);
		    		bAnniversaire.setAnnee(datePicker.getYear());
		    		bAnniversaire.setMois(datePicker.getMonth()+1);
		    		bAnniversaire.setJour(datePicker.getDayOfMonth());
		    		bAnniversaire.setAnneeConnue(checkBoxAnnee.isChecked() ? false:true);
		    		bAnniversaire.setAutreEven(checkBoxAutreEven.isChecked());
		    		daoAnniversaires.ecrireAnniversaire(bAnniversaire);
		    		
		    		sqLiteAnniv.closeDB (idSqLite);
		    		
		    		// Relance du service	    		
		    	    Intent intentServiceAutoRun = new Intent(CreationEventActivity.this, ServiceAutoRun.class);
		    	    intentServiceAutoRun.putExtra("CHANGE", true);
		    		startService(intentServiceAutoRun);
		    		
		        	finish();
		        }
	    	break;

	    	case R.id.buttonAnnuler:
	    		finish();
	    	break;
	    	
	    	case R.id.checkBoxAutreEven:
	    		bAnniversaire.setAutreEven(!bAnniversaire.getAutreEven());
	    		modeAnnivAutre(bAnniversaire.getAutreEven());
	    	break;
	    	
	    	case R.id.checkBoxYearUnknow:
	    		bAnniversaire.setAnneeConnue(!bAnniversaire.getAnneeConnue());
	    		setVisibiltyYear(bAnniversaire.getAnneeConnue());
	    	break;
    	}
    }
	
	@Override
	protected void onActivityResult(int requestCode, int resultCode, Intent data) {
	    if (data != null) {
	        Uri uri = data.getData();

	        if (uri != null) {
	            Cursor c = null;
	            try {
	            	// Rcupration de l'ID du contact slectionn
	                c = getContentResolver().query(uri, new String[]{ 
	                			ContactsContract.CommonDataKinds.Phone._ID},
	                        null, null, null);

	                Boolean resultat=c.moveToFirst();
	                if (LogAnniv.isDebug())
	                {
	                	LogAnniv.d(CLASSE, "Resultat recherche ID="+resultat);
	                }
	                if (c != null && resultat) {
	                	int id= c.getInt(0);
	                	LogAnniv.d(CLASSE, "ID="+id);
	                	
                		TraiterContact(id);
	                }
	            } finally {
	                if (c != null) {
	                    c.close();
	                }
	            }
	        }
	    }
	}

	public void TraiterContact(long id) {
//	    Toast.makeText(this, "id:"+id, Toast.LENGTH_LONG).show();
		
		// On mmorise cet id
		idContact = id;
		bAnniversaire.setIdContact(idContact);
		
		ContactAnniv contactAnniv=new ContactAnniv (id, getContentResolver());
		
		if (!bAnniversaire.getAutreEven()) // Si anniversaire contact, prsenter son nom ?
		{
	        final String given = contactAnniv.getPrenom();
	        final String family = contactAnniv.getNom();
		        
	        if (LogAnniv.isDebug())
	        {
	        	LogAnniv.d(CLASSE, "given:"+given+", family:" + family);
	        }
	        
	        Boolean bChange=false; // Pour le moment, pas de changement nom/prenom entre la saisie et le contact rapatri
	        // On compare avec les champs de saisie
	        String prenomSaisi=editTextPrenom.getText().toString();
	        if (prenomSaisi.isEmpty())
	        {
	        	if (given !=null)
	        	{
	        		bAnniversaire.setPrenom(given);
	        	}
	        	else
	        	{
	        		bAnniversaire.setPrenom("");
	        	}
	        	editTextPrenom.setText(bAnniversaire.getPrenom());
	        }
	        else
	        {
	        	if (given != null && prenomSaisi.compareTo(given)!=0)
	        	{
	        		bChange=true;
	        	}
	        }
	        
	        String nomSaisi=editTextNom.getText().toString();
	        if (nomSaisi.isEmpty())
	        {
	        	if (family!=null)
	        	{
	        		bAnniversaire.setNom(family);
	        	}
	        	else
	        	{
	        		bAnniversaire.setNom("");
	        	}
	        	editTextNom.setText (bAnniversaire.getNom());
	        }
	        else
	        {
	        	if (family != null && nomSaisi.compareTo(family)!=0)
	        	{
	        		bChange=true;
	        	}
	        }
	        
		    // Si diffrence entre la saisie et le contact rapatri, proposer d'apporter 
		    // les modifications automatiquement
		    if (bChange)
		    {
	            AlertDialog.Builder mesAlert = new AlertDialog.Builder(this);
	            
	            mesAlert.setTitle(R.string.alert_title_indetite_diff);
	            mesAlert.setMessage(R.string.alert_mes_infos_ident_diff);
	            // Dclaration des boutons  utiliser
	            mesAlert.setNeutralButton(R.string.alert_btn_indetite_diff_Conserver, new DialogInterface.OnClickListener() {
	            public void onClick(DialogInterface dlg, int sumthin) {
	            	// Pour le moment, rien  faire !
	            }});
	            mesAlert.setPositiveButton(R.string.alert_btn_indetite_diff_Maj, new DialogInterface.OnClickListener() {
	            public void onClick(DialogInterface dlg, int sumthin) {
            		bAnniversaire.setPrenom(formaterString (given));
            		bAnniversaire.setNom(formaterString (family));
	            	editTextPrenom.setText(bAnniversaire.getPrenom());
	        		editTextNom.setText (bAnniversaire.getNom());
	            }

				private String formaterString(String chaine) {
					String libelle;
					
					if (chaine==null)
					{
						libelle="";
					}
					else
					{
						// On supprime tous les espaces (devant et derriere)
						libelle=chaine.trim();
						
						// Puis on met la premire lettre en majuscule
						char[] char_table = libelle.toCharArray();
						if (char_table.length>0) // Et oui, il faut au moins un caractre  traiter !
						{
							char_table[0]=Character.toUpperCase(char_table[0]);
							libelle = new String(char_table);
						}
					}
					return libelle;
				}});
	            
	            // Affichage du message
	            mesAlert.show();
		    }
	        
	        // Maj de l'image du bouton
	        afficherPhotoContactBouton(contactAnniv);
		}	    
	}
	
	// Affichage de la photo du contact sur le bouton de rcupration d'un contact
	private void afficherPhotoContactBouton(ContactAnniv contactAnniv)
	{
		InputStream photo=contactAnniv.openPhoto();
        if (photo!=null)
        {
        	Drawable buttonImg = Drawable.createFromStream(photo, null);
        	if (buttonImg!=null)
        	{
	        	buttonAssocier.setImageDrawable(buttonImg);
        	}
        }
        else
        {
        	buttonAssocier.setImageResource(R.drawable.ic_unknow);
        }
	}
	
	// Si l'anne est connu, afficher l'anne dans le date picker
	private void setVisibiltyYear (boolean visibility)
	{
	    try {
	    	Field f[] = datePicker.getClass().getDeclaredFields();
	    	for (Field field : f) {
	    		LogAnniv.d(CLASSE, "Field="+field.getName());
	    	    if (field.getName().equals("mYearPicker") || field.getName().equals("mYearSpinner")) {
	    	        field.setAccessible(true);
	    	        Object yearPicker = new Object();
	    	        yearPicker = field.get(datePicker);
	    	        
	    	        if (visibility)
	    	        {
	    	        	((View) yearPicker).setVisibility(View.VISIBLE);
	    	        }
	    	        else
	    	        {
	    	        	((View) yearPicker).setVisibility(View.GONE);
	    	        }
	    	    }
	    	}
	    } catch (SecurityException e) {
	    	LogAnniv.d(CLASSE, e.getMessage());
	    } 
	    catch (IllegalArgumentException e) {
	    	LogAnniv.d(CLASSE, e.getMessage());
	    } catch (IllegalAccessException e) {
	    	LogAnniv.d(CLASSE, e.getMessage());
	    }
	}
	
}

