package fr.free.supertos.anniversaire;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOError;
import java.io.IOException;
import java.util.List;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.w3c.dom.Document;
import org.xml.sax.SAXException;
import org.xmlpull.v1.XmlSerializer;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.pm.ActivityInfo;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.os.Bundle;
import android.util.Xml;
import android.view.Menu;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TextView;
import android.widget.Toast;
import fr.free.supertos.anniversaire.bean.BeanAnniversaires;
import fr.free.supertos.anniversaire.bean.BeanEtat;
import fr.free.supertos.anniversaire.bean.BeanParametres;
import fr.free.supertos.anniversaire.dao.DaoAnniversaires;
import fr.free.supertos.anniversaire.dao.DaoEtat;
import fr.free.supertos.anniversaire.dao.DaoParametres;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;
import fr.free.supertos.anniversaire.service.ServiceAutoRun;
import fr.free.supertos.anniversaire.xml.XmlAnniversaire;

/**
 * Point d'entre UI de l'application anniversaire
 * Permet  l'utilisateur d'importer ou d'exporter les donnes de la base
 * 
 * @author supertos
 *
 */
public class ImportExportActivity extends Activity implements OnClickListener{

	private final static String CLASSE = ImportExportActivity.class.getName();
	
	// Les enregistrements qui alimenteront la liste des lments de la liste
	private List<BeanAnniversaires> listAnniversaires;
	private SqLiteAnniversaire sqLiteAnniv=null;
	private SQLiteDatabase idSqLite=null;
	
	private Context baseContext;

	private ThreadImportExportXML maTache;
	
	// Constantes
	public final static Integer IMPORT = 1;
	public final static Integer EXPORT = 2;

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_importexport);
		
		// Forcer l'orientation  portrait
		setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_PORTRAIT);
		 
		TextView textTexte = (TextView)findViewById(R.id.textTexte);
		TextView textRepertoire = (TextView)findViewById(R.id.textRepertoire);
		
		Button buttonImport = (Button)findViewById(R.id.buttonImport);
		buttonImport.setOnClickListener((OnClickListener) this);
		Button buttonExport = (Button)findViewById(R.id.buttonExport);
		buttonExport.setOnClickListener((OnClickListener) this);
		Button buttonAnnuler = (Button)findViewById(R.id.buttonAnnuler);
		buttonAnnuler.setOnClickListener((OnClickListener) this);
				  
		// Testons la prsence d'une carte SD et sa disponibilit.
		XmlAnniversaire xml=new XmlAnniversaire();
		File path=null;
		try
		{
			path = xml.detecterCarteSD(this);
			textTexte.setText(R.string.texte_import_export);
		} catch (IOError e) {
			// Pour l'internationalisation, nous ne pouvons pas afficher le teste directement, mais devons passer par la ressource String
			if (LogAnniv.isDebug())
			{
				LogAnniv.d(CLASSE, "Exception:" + e.getMessage());
			}
			if (e.getMessage().compareTo("java.lang.Throwable: La carte SD n'est pas monte")==0)
			{
				// Pb. sur carte SD
				textTexte.setText(R.string.texte_import_export_no_sd);
				buttonImport.setEnabled(false);
				buttonExport.setEnabled(false);
			}
			else
			{
				// Pb. pour accder au rpertoire anniversaire
				textTexte.setText(R.string.texte_import_export_pb_io);
				buttonImport.setEnabled(false);
				buttonExport.setEnabled(false);				
			}
		}
		if (path != null)
		{
			textRepertoire.setText(path.getAbsolutePath());
		}
		else
		{
			textRepertoire.setText("");
		}
	}
	
	// Exporter les tables de la base dans des fichiers XML (1 fichier par table)
	private void exportXML(Context contexte) {
		// TODO Auto-generated method stub
		XmlAnniversaire xml=new XmlAnniversaire();
		File repertoireAnniversaire = xml.createXMLFile(contexte);
		XmlSerializer serializer = Xml.newSerializer();
		
		sqLiteAnniv = new SqLiteAnniversaire();
		idSqLite=sqLiteAnniv.openDB(this);
		
		try {
			
			FileOutputStream fileos = new FileOutputStream(repertoireAnniversaire.getAbsolutePath() + "/anniversaire.xml");

			serializer.setOutput(fileos, "UTF-8");
			serializer.startDocument(null,	Boolean.valueOf(true));
			serializer.startTag(null, "anniversaire");
			
			DaoAnniversaires daoAnniversairesInput=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
			DaoAnniversaires daoAnniversairesOutput=(DaoAnniversaires)DaoFactory.daoAnniversaires(serializer);
			
			listAnniversaires=daoAnniversairesInput.listAnniversaires();
			
			for (BeanAnniversaires element:listAnniversaires)
			{
				daoAnniversairesOutput.ecrireAnniversaire(element);
			}
			
			serializer.endDocument();
			serializer.flush();
			fileos.close();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		try {
			
			FileOutputStream fileos = new FileOutputStream(repertoireAnniversaire.getAbsolutePath() + "/parametres.xml");

			serializer.setOutput(fileos, "UTF-8");
			serializer.startDocument(null,	Boolean.valueOf(true));
			serializer.startTag(null, "parametres");
			
			DaoParametres daoParametresInput=(DaoParametres)DaoFactory.daoParametres(idSqLite);
			DaoParametres daoParametresOutput=(DaoParametres)DaoFactory.daoParametres(serializer);
			
			BeanParametres bParametres=daoParametresInput.getParametres();
			
			daoParametresOutput.ecrireParametres(bParametres);

			serializer.endDocument();
			serializer.flush();
			fileos.close();
		} catch (IllegalArgumentException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IllegalStateException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		idSqLite.close();
		xml.closeXMLFile();	
	}

	// Importer les flux xml dans la base (1 fichier par table)
	private void importXML()
	{
		XmlAnniversaire xml=new XmlAnniversaire();
		XmlSerializer serializer = Xml.newSerializer();
		// Maintenant lecture !
		File repertoireAnniversaireInput = xml.openXML(this);
		
		sqLiteAnniv = new SqLiteAnniversaire();
		idSqLite=sqLiteAnniv.openDB(this);
		
		DocumentBuilder docBuilder;
		try {
			
			DocumentBuilderFactory docBuilderFactory = DocumentBuilderFactory.newInstance();
            docBuilder = docBuilderFactory.newDocumentBuilder();
            
            // Traitement du fichier anniversaire.xml
            File ficAnniv=new File (repertoireAnniversaireInput+"/anniversaire.xml");
           
			try {
				Document doc = docBuilder.parse (ficAnniv);
				DaoAnniversaires daoAnniversairesInput=(DaoAnniversaires)DaoFactory.daoAnniversaires(serializer);
				doc.getDocumentElement ().normalize ();
				List<BeanAnniversaires> listAnniversaires = daoAnniversairesInput.listAnniversaires(doc);
				// Il faut maintenant intgrer les beans dans la base...
				DaoAnniversaires daoAnniversairesOuput=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
				//Effaons pour commencer tout ce qu'il y avait dans la table
				daoAnniversairesOuput.toutEffacer();
				if (listAnniversaires!=null)
				{
					for (BeanAnniversaires element:listAnniversaires)
					{
						LogAnniv.d("test", "Ecriture bean" + element + " vers " + idSqLite);
						element.setId(0); // Oblig, car l'Id est en autoincrement. Or il risque d'avoir conflit entre l'ancien id et un nouvel enregistrement portant le mme id.
						daoAnniversairesOuput.ecrireAnniversaire(element);
					}
				}
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SAXException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			
            // Traitement du fichier parametres.xml
            File ficParam=new File (repertoireAnniversaireInput+"/parametres.xml");
           
			try {
				Document doc = docBuilder.parse (ficParam);
				DaoParametres daoParametresInput=(DaoParametres)DaoFactory.daoParametres(serializer);
				doc.getDocumentElement ().normalize ();
				BeanParametres bParametres = daoParametresInput.getParametres(doc);
				
				// Maintenant, on met  jour les paramtres en base
				DaoParametres daoParametresOuput=(DaoParametres)DaoFactory.daoParametres(idSqLite);
				daoParametresOuput.ecrireParametres(bParametres);
				// Intgration des paramtres dans la base...
			} catch (FileNotFoundException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (SAXException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}

			// Traitement de la table etat pour indiquer que les alarmes ne se sont pas dclenches aujourd'hui.
			DaoEtat daoEtat=(DaoEtat)DaoFactory.daoEtat(idSqLite);
			BeanEtat bEtat=new BeanEtat();
			bEtat.setStartAMS(0);
			daoEtat.ecrireEtat(bEtat);
			
			// Et relance du service	    		
    	    Intent intentServiceAutoRun = new Intent(ImportExportActivity.this, ServiceAutoRun.class);
    	    intentServiceAutoRun.putExtra("CHANGE", true);
    		startService(intentServiceAutoRun);
		} catch (ParserConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
		sqLiteAnniv.closeDB(idSqLite);
		xml.closeXMLFile();
	}
	
	protected void onResume()
	{
	
		super.onResume();
		
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Resume !!!");
    	}
		
		baseContext=this.getBaseContext();
	}
	
	protected void onDestroy()
	{
		super.onDestroy();
		
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Destroy !!!");
    	}
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}
	
	public void onClick(View view)
	{
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Clique !!!");
    	}
    	switch (view.getId())
    	{	    	
	    	case R.id.buttonImport:
	    		maTache=new ThreadImportExportXML(this);
	    		maTache.execute(IMPORT); 
	    	break;

	    	case R.id.buttonExport:
	    		maTache=new ThreadImportExportXML(this);
	    		maTache.execute(EXPORT);
	    	break;

	    	case R.id.buttonAnnuler:
	    		finish();
	    	break;	    	
    	}
    }
    
    class ThreadImportExportXML extends AsyncTask<Integer, Long, Void> {

		private boolean importXML;
		private Context context;

		public ThreadImportExportXML(Activity activity) {
			this.context = activity.getApplicationContext();
		}
		@Override
		protected void onPreExecute() {

		}

		@Override
		protected void onProgressUpdate(Long... values){
			// Rien  faire...
		}

		@Override
		protected Void doInBackground(Integer... arg0) {

			if (arg0.length == 1) {
		        
				if (arg0[0]==IMPORT)
				{
					importXML=true;
					importXML();
				}
				else
				{
					importXML=false;
					exportXML(this.context);
				}
	        }
			else
			{
				throw  new IllegalArgumentException("Nombre d'arguments incorrects. Il faut prciser IMPORT ou EXPORT");
			}
			
			return null;
		}

		@Override
		protected void onPostExecute(Void result) {
			
			Toast toast;
			if (importXML==true)
			{
				toast = Toast.makeText(baseContext, R.string.texte_import_fin, Toast.LENGTH_LONG);
			}
			else
			{
				toast = Toast.makeText(baseContext, R.string.texte_export_fin, Toast.LENGTH_LONG);
			}
		
			toast.show();
		}
	}
}
