package fr.free.supertos.anniversaire;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.CheckBox;
import android.widget.ImageButton;
import android.widget.ListView;
import android.widget.TextView;
import fr.free.supertos.anniversaire.bean.BeanAnniversaires;
import fr.free.supertos.anniversaire.classes.MyListAdapter;
import fr.free.supertos.anniversaire.constantes.ConstantesMyListAdapter;
import fr.free.supertos.anniversaire.dao.DaoAnniversaires;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;

/**
 * Lister l'ensemble des vnements prsent dans la base
 * Permettre la cration de nouveaux, la suppression
 * 
 * @author supertos
 *
 */
public class ListEventActivity extends ListActivity implements OnClickListener {

	private final static String CLASSE = ListEventActivity.class.getName();
	
	// Les enregistrements qui alimenteront la liste des lments de la liste
	private List<BeanAnniversaires> listAnniversaires=null;
	
	private SqLiteAnniversaire sqLiteAnniv=null;
	private SQLiteDatabase idSqLite=null;
	
	private ListView listView;
	private ImageButton buttonCreer;
	private ImageButton buttonSupprimer;
	private ImageButton buttonBack;
	private ImageButton buttonTrash;

	private TextView textListe;	
	
	private MyListAdapter monAdaptater; // Adaptateur de la liste

	private boolean bSuppression;
	
	private ThreadChargerListAniversaires maTache;

	private Context baseContext;

	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_liste);

		// Recherche des widgets
		listView=(ListView)findViewById(android.R.id.list);
		
		buttonCreer=(ImageButton)findViewById(R.id.buttonCreer);
		buttonCreer.setOnClickListener((OnClickListener) this);
		
		buttonSupprimer=(ImageButton)findViewById(R.id.buttonSupprimer);
		buttonSupprimer.setOnClickListener((OnClickListener) this);
		
		buttonBack=(ImageButton)findViewById(R.id.buttonBack);
		buttonBack.setOnClickListener((OnClickListener) this);
		
		buttonTrash=(ImageButton)findViewById(R.id.buttonPoubelle);
		buttonTrash.setOnClickListener((OnClickListener) this);

		textListe=(TextView)findViewById(R.id.textListe);
	}

	protected void onResume()
	{
	
		super.onResume();
		
		// Connexion  la base de donnes
		sqLiteAnniv = new SqLiteAnniversaire();
		idSqLite=sqLiteAnniv.openDB(this);
		                
		baseContext=this.getBaseContext();
		
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Resume !!!");
    	}
		
		if (listAnniversaires==null)
		{
			genererListAnniversaire();
		}
	}
	
	private void genererListAnniversaire() {
		// Rcuprer la liste des vnements peut prendre du temps, donc pour viter un ANR, lancement dans un thread
		maTache=new ThreadChargerListAniversaires();
		maTache.execute();
	}

	class ThreadChargerListAniversaires extends AsyncTask<Void, Long, Void> {

		ProgressDialog dialogWait;

		@Override
		protected void onPreExecute() {
			dialogWait = ProgressDialog.show(ListEventActivity.this, getString (R.string.info_recup_liste), getString (R.string.info_patientez), true);
		}

		@Override
		protected void onProgressUpdate(Long... values){
			// Rien  faire...
		}

		@Override
		protected Void doInBackground(Void... arg0) {

			DaoAnniversaires daoAnniversaires=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
			
			listAnniversaires=daoAnniversaires.listAnniversaires();
				
			return null;
		}

		@Override
		protected void onPostExecute(Void result) {
			// Cration de la ArrayList qui nous permettra de remplir la listView
			ArrayList<HashMap<String, String>> listItem = new ArrayList<HashMap<String, String>>();

			if (!listAnniversaires.isEmpty())
			{

				// On dclare la HashMap qui contiendra les informations pour un item
				HashMap<String, String> map;

				for (BeanAnniversaires element:listAnniversaires)
				{
					map = new HashMap<String, String>();
					map.put("identification", element.getPrenom() + " " + element.getNom());
					listItem.add(map);
				}
				enableTrash(true);
				String texte=getResources().getString(R.string.nbEvents);
				textListe.setText(texte + listAnniversaires.size());
			}
			else
			{
				// Il n'y a plus rien  supprimer !
				faireButtonBack();
				// Dsactiver la poubelle
				enableTrash(false);
				textListe.setText(R.string.noEvents);
			}
			monAdaptater = new MyListAdapter(baseContext, listItem,
					R.layout.list_detail, new String[] { "identification" }, new int[] {
							R.id.identification });

			// On attribue  notre listView l'adaptateur que l'on vient de crer
			listView.setAdapter(monAdaptater);

			// On peut fermer la dialogue permettant de faire patienter l'utilisateur...
			dialogWait.dismiss();

		}
	
		@Override
		protected void onCancelled() {
		}
	}

	protected void onPause()
	{
		super.onPause();
		
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Pause !!!");
    	}
		// Fermeture accs base
		if (sqLiteAnniv!=null && idSqLite!=null)
			sqLiteAnniv.closeDB (idSqLite);
	}

	protected void onDestroy()
	{
		super.onDestroy();
		
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Destroy !!!");
    	}
	}
	
	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.listevent, menu);
		return true;
	}

	/**
	 * Listener sur ListView affichant les anniversaires en cours
	 */
	// Listener sur ListView
    protected void onListItemClick(ListView parent, View v, int position, long id) {
    	if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "2 - view" + v);
    	}
    	// La position de l'item correspond  la position de notre bean, profitons en !
    	// Toast.makeText(ListEventActivity.this,"ListItemClick:" + position, Toast.LENGTH_SHORT).show();
    	
    	final BeanAnniversaires bAnniversaire=listAnniversaires.get(position);
        runCreationEventActivity(bAnniversaire.getId());
    }
    
    /**
     * Listener sur les widgets OnClick (boutons ici)
     */
    // Mthode onClick de OnClickListener
    public void onClick(View view){
    	//Toast.makeText(ListEventActivity.this,"onClick", Toast.LENGTH_SHORT).show();
    	if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "1 - view" + view);
    	}
    	switch (view.getId())
    	{
	    	case R.id.buttonCreer:
        		// Toast.makeText(ListEventActivity.this,"Cration d'un nouveau profil", Toast.LENGTH_SHORT).show();
        		runCreationEventActivity ();
	    	break;
	    	
	    	case R.id.buttonSupprimer:
	    		if (!bSuppression)
	    		{
	    			// Toast.makeText(ListEventActivity.this,"Suppression d'un nouveau profil", Toast.LENGTH_SHORT).show();
	    			bSuppression=true;
	    			monAdaptater.showCb();
	    			
	    			buttonSupprimer.setVisibility(View.INVISIBLE);
	    			buttonCreer.setVisibility(View.INVISIBLE);

	    			buttonTrash.setVisibility(View.VISIBLE);
	    			buttonBack.setVisibility(View.VISIBLE);
	    		}
	    	break;
	    	
	    	case R.id.buttonBack:
	    		faireButtonBack();
	    	break;

	    	case R.id.buttonPoubelle:
	    		if (bSuppression)
	    		{
	    			SupprimerProfils();
	    		}
    	}
    }  
    
    // Oprations qui sont  effectuer pour faire un retour  l'affichage prcdent des boutons
    private void faireButtonBack() {
		if (bSuppression)
		{
			bSuppression=false;
			monAdaptater.hideCb();
			
			buttonSupprimer.setVisibility(View.VISIBLE);
			buttonCreer.setVisibility(View.VISIBLE);

			buttonTrash.setVisibility(View.INVISIBLE);
			buttonBack.setVisibility(View.INVISIBLE);
		}
	}

	private void enableTrash(boolean clickable) {
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "enableTrash:" + clickable);
    	}
		buttonSupprimer.setClickable(clickable);
	}

	private void SupprimerProfils() {
		
    	// Cration d'un message d'alerte
        AlertDialog.Builder mesAlert = new AlertDialog.Builder(this);
        
        mesAlert.setTitle(R.string.alert_title_supprimer_profil);
        
    	if (monAdaptater.auMoins1CbCoche()==ConstantesMyListAdapter.COCHE)
    	{
	    	
	        mesAlert.setMessage(R.string.alert_title_supprimer_profil_mess);
	        // Dclaration des boutons  utiliser
	        mesAlert.setNeutralButton(R.string.alert_title_btn_cancel, null);
	        mesAlert.setPositiveButton(R.string.alert_title_btn_oui, new DialogInterface.OnClickListener() {
	        public void onClick(DialogInterface dlg, int sumthin) {
	        	int nombreComboBox = monAdaptater.getNombreCb();
	        	
	        	if (nombreComboBox > 0)
	        	{
	        		for (Integer i=0; i<nombreComboBox; i++)
	        		{
	        			// On ne vire que les enregistrements dont les combobox sont cochs.
	        			if (monAdaptater.listEtatCb.get(i) == ConstantesMyListAdapter.COCHE)
	        			{
	        				// Comme le num d'ordre des comb et le mme que celui de la liste des bg
	        				// Rcupration du bg dont le n d'ordre = celui de la combo
	        				BeanAnniversaires bgAnniversaire = listAnniversaires.get(i);
	        				
	        				if (LogAnniv.isDebug())
	        		    	{
	        		    		LogAnniv.d(CLASSE, "Suppression du record id="+bgAnniversaire.getId());
	        		    	}
	        				// Suppression de l'enregistrement en utilisant l'id dclar dans le bg
	        		    	DaoAnniversaires daoAnniversaires=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
	        		    	daoAnniversaires.supprimerAnniversaire(bgAnniversaire.getId());
	        			}
	        		}
	        		
	        		// Regenration de la liste des bg et maj de l'adaptateur
	        		genererListAnniversaire();
/*	        		bSuppression=true;
	        		monAdaptater.showCb();*/
	        		
		    		// Et on referme la poubelle
		    		faireButtonBack();
	        	}
	        }});        
	        mesAlert.setNegativeButton(R.string.alert_title_btn_non, null);
    	}
    	else
    	{
    		mesAlert.setMessage(R.string.alert_title_supprimer_profil_mess_rien_a_faire);
    		mesAlert.setPositiveButton(android.R.string.ok , null);
    	}
    	
        // Affichage du message d'alerte
        mesAlert.show();

	}

    /**
     * Lancement d'une activit permettant la cration/modification d'un vnement 
     * 
     * @param id id de l'vnement  modifier ou -1 pour une cration
     */
	protected void runCreationEventActivity (int id)
    {
//		Toast.makeText(ListEventActivity.this,"Cration event", Toast.LENGTH_SHORT).show();
		Intent intentEventCreer=new Intent (ListEventActivity.this, CreationEventActivity.class);
		
		if (id!=-1)
		{
			intentEventCreer.putExtra("id", id);
		}
		startActivity(intentEventCreer);    	
    }
    
	/**
	 * Lancement d'une mthode nomme runCreationEventActivity (int id) en forant id  -1 pour permettre la cration d'un vnement
	 */
    protected void runCreationEventActivity ()
    {
    	runCreationEventActivity (-1);
    }
    
    /**
     * checkBoxClick permet d'indiquer que pour vnement slectionn, il faudra le marquer comme coch (si prcdemment dcoch) ou dcoch (si coch) pour une ventuelle suppression
     * 
     * @param v
     */
    public void checkBoxClick(View v) {
//    	Toast.makeText(ListEventActivity.this,"checkBoxClick:", Toast.LENGTH_SHORT).show();
		CheckBox cb = (CheckBox) v;
		
		if (cb==null || cb.getTag()==null)
		{
			if (LogAnniv.isDebug())
	    	{
	    		LogAnniv.d(CLASSE, "checkBoxClick:" + cb.getTag().toString());
	    	}
		}	
		//on rcupre la position  l'aide du tag dfini dans la classe MyListAdapter
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "checkBoxClick:" + cb.getTag().toString());
    	}
		int position = Integer.parseInt(cb.getTag().toString());
		
		// On rcupre l'lment sur lequel on va changer la couleur
		monAdaptater.changeEtatCb (position);
	}
    
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	     
	    switch (item.getItemId()) {
	    
	        case R.id.action_param_alarm:
	        	Intent intentEventAlarm=new Intent (ListEventActivity.this, ParamActivityAlarm.class);
	            startActivity(intentEventAlarm);
	            return true;
	        case R.id.action_param_rappel:
	        	Intent intentEventRappel=new Intent (ListEventActivity.this, ParamActivityRappel.class);
	            startActivity(intentEventRappel);
	            return true;
	        case R.id.action_param_notif:
	        	Intent intentEventNotif=new Intent (ListEventActivity.this, ParamActivityNotif.class);
	            startActivity(intentEventNotif);
	            return true;            
	        case R.id.action_a_propos_de:
	        	Intent intentEventAProposDe=new Intent (ListEventActivity.this, AProposDeActivity.class);
	            startActivity(intentEventAProposDe);
	            return true;
	        case R.id.action_param_import_export:
	        	Intent intentImpExp=new Intent (ListEventActivity.this, ImportExportActivity.class);
	            startActivity(intentImpExp);
	            return true;
	        default:
	            return super.onOptionsItemSelected(item);
	    }
	}
}
