package fr.free.supertos.anniversaire;

import java.util.Calendar;
import java.util.List;

import android.app.AlertDialog;
import android.app.ListActivity;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.res.Resources;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.os.AsyncTask;
import android.os.Bundle;
import android.provider.ContactsContract;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.ListView;
import android.widget.TextView;
import fr.free.supertos.anniversaire.bean.BeanAnniversaires;
import fr.free.supertos.anniversaire.constantes.ConstantesAnniversaires;
import fr.free.supertos.anniversaire.dao.DaoAnniversaires;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;
import fr.free.supertos.anniversaire.service.ServiceAutoRun;

/**
 * Point d'entre UI de l'application anniversaire
 * Va lister l'ensemble des anniversaires  venir, ou l'ensemble des anniversaires pour ce jour
 * 
 * @author supertos
 *
 */
public class MainActivity extends ListActivity {

	private final static String CLASSE = MainActivity.class.getName();
	
	// Pour grer la liste des anniversaires:
	// Les lments de la liste
	private String[] items; 
	// Les enregistrements qui alimenteront la liste des lments de la liste
	private List<BeanAnniversaires> listAnniversaires;
	private List<BeanAnniversaires> listFutursAnniversaires;
	
	private SqLiteAnniversaire sqLiteAnniv=null;
	private SQLiteDatabase idSqLite=null;
	
	private ListView listView;
	private TextView textViewListAnniversaire;
	private TextView textViewPasAnniv;
	private TextView textViewProchainAnniv;
	
	private ArrayAdapter<String> monAdaptater;

	private Context baseContext;

	private fr.free.supertos.anniversaire.MainActivity.ThreadChargerListAnniversaires maTache;

	
	
	@Override
	/**
	 * onCreate std
	 */
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		// Recherche des widgets
		listView=getListView(); //(ListView)findViewById(android.R.id.list);
		textViewListAnniversaire=(TextView)findViewById(R.id.textViewListAnniversaire);
		textViewPasAnniv=(TextView)findViewById(R.id.textViewPasAnniv);
		textViewProchainAnniv=(TextView)findViewById(R.id.textViewProchainAnniv);
		
		// Lancement du service si celui-ci n'a pas dmarr les alarmes suite au boot !
	    Intent intentServiceAutoRun = new Intent(MainActivity.this, ServiceAutoRun.class);
		startService(intentServiceAutoRun);
	}

	/**
	 * onResume std
	 */
	protected void onResume()
	{
	
		super.onResume();
		
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Resume !!!");
    	}
		
		baseContext=this.getBaseContext();
		
		// Rcupration de la liste des vnements
		genererListAnniversaire();
	}
	
	/**
	 * Rcupration de la liste des vnements:
	 * - Soit la liste des anniversaires en cours
	 * - Soit la liste des futures anniversaires si pas d'anniversaire en cours.
	 * 
	 * Note: Lancement de la rcupration dans un thread afin d'viter un ANR.
	 */
	private void genererListAnniversaire() {
		// Rcuprer la liste des vnements peut prendre du temps, donc pour viter un ANR, lancement dans un thread
		maTache=new ThreadChargerListAnniversaires();
		maTache.execute();
	}
	
	/**
	 * Destruction de l'activit
	 */
	protected void onDestroy()
	{
		// Rien de spcial  faire ici...
		super.onDestroy();
		
		if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "Destroy !!!");
    	}
	}
	
	@Override
	/**
	 * onCreateOptionsMenu std
	 */
	public boolean onCreateOptionsMenu(Menu menu) {
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	/**
	 * Mthode std permettant de traiter l'option de menu slectionne
	 */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
	     
	    switch (item.getItemId()) {
	    
	    	case R.id.action_settings:
	    		runListEventActivity();
	    		return true;
	        case R.id.action_param_alarm:
	        	Intent intentEventAlarm=new Intent (MainActivity.this, ParamActivityAlarm.class);
	            startActivity(intentEventAlarm);
	            return true;
	        case R.id.action_param_rappel:
	        	Intent intentEventRappel=new Intent (MainActivity.this, ParamActivityRappel.class);
	            startActivity(intentEventRappel);
	            return true;
	        case R.id.action_param_notif:
	        	Intent intentEventNotif=new Intent (MainActivity.this, ParamActivityNotif.class);
	            startActivity(intentEventNotif);
	            return true;   
	        case R.id.action_param_import_export:
	        	Intent intentImpExp=new Intent (MainActivity.this, ImportExportActivity.class);
	            startActivity(intentImpExp);
	            return true;
	        case R.id.action_a_propos_de:
	        	Intent intentEventAProposDe=new Intent (MainActivity.this, AProposDeActivity.class);
	            startActivity(intentEventAProposDe);
	            return true;
	            	            
	        default:
	            return super.onOptionsItemSelected(item);
	    }
	}
	
	/**
	 * Listener sur ListView affichant les anniversaires en cours ou la liste des futures anniversaires
	 */
	// Listener sur ListView
    protected void onListItemClick(ListView parent, View v, int position, long id) {
    	// La position de l'item correspond  la position de notre bean, profitons en !
    	if (LogAnniv.isDebug())
    	{
    		LogAnniv.d(CLASSE, "click sur liste ..." + v);
    	}

    	final BeanAnniversaires bAnniversaire;
    	
    	// La liste qui est affiche correspond-t-elle  la liste des anniversaires pour ce jour
    	if (!listAnniversaires.isEmpty())
    	{
    		// Oui, c'est la liste des anniversaires.
    		// Dans ce cas, il est peut-tre possible de lancer l'activit de consultation des informations d'un contact
    		// qui permettra ventuellement de l'appeler ou d'envoyer un sms.
    		bAnniversaire=listAnniversaires.get(position);
    	
    		// Le contact est-il connu ?
	    	if (bAnniversaire.getIdContact()==ConstantesAnniversaires.CONTACTINCONNUE)
	    	{
	    		// Pas de contact dfini, donc pas d'information sur le moyen de le contacter !
	    		// Cration d'un message d'alerte qui va demander s'il faut ou non assoicer un contact...
	            AlertDialog.Builder mesAlert = new AlertDialog.Builder(this);
	            
	            mesAlert.setTitle(R.string.alert_title_no_contact);
	            mesAlert.setMessage(R.string.alert_mes_no_contact);
	            // Dclaration des boutons  utiliser
	            mesAlert.setNeutralButton(R.string.alert_btn_no_contact_fermer, new DialogInterface.OnClickListener() {
	            public void onClick(DialogInterface dlg, int sumthin) {
	            	if (LogAnniv.isDebug())
	            	{
	            		LogAnniv.d(CLASSE, "Bouton Fermer press");
	            	}
	            }});
	            mesAlert.setPositiveButton(R.string.alert_btn_no_contact_Associer, new DialogInterface.OnClickListener() {
	            	// Sur ce bouton, il n'y a rien  faire pour le moment !
	            public void onClick(DialogInterface dlg, int sumthin) {
	            	if (LogAnniv.isDebug())
	            	{
	            		LogAnniv.d(CLASSE, "Bouton Associer press");
	            	}
	            	
	            	// Une association vnement/contact est demande
	            	// Lancement d'une mthode qui va  son tour lancer une activit permettant de choisir un contact et de l'associer  l'vnement.
	            	runCreationEventActivity(bAnniversaire.getId());
	            }});
	            
	            // Affichage du message
	            mesAlert.show();
	            
	    	}
	    	else
	    	{
	    		// L'id contact etant connu, lancement de l'cran de consultation des informations d'un contact.
	    		// Ce qui permettra  l'utilisateur d'appeler cette personne, de lui envoyer un sms, ...
	    		Intent intentContact = new Intent(Intent.ACTION_VIEW, Uri.withAppendedPath(ContactsContract.Contacts.CONTENT_URI, ""+ bAnniversaire.getIdContact()));
	    		startActivity(intentContact); 
	    	}
    	}
    	else
    	{
    		// Non, il ne s'agit pas de la liste des anniversaire de ce jour, mais de la liste des anniversaires  venir...
    		// Donc, si clique sur un lment de la liste des futures anniversaires.
    		// Rcupration des informations sur l'vnement correspondant.
    		bAnniversaire=listFutursAnniversaires.get(position);

        	if (LogAnniv.isDebug())
        	{
            	
                    	if (LogAnniv.isDebug())
            	{
	        		LogAnniv.d(CLASSE, "position:" + position);
	        		LogAnniv.d(CLASSE, "nom:" + bAnniversaire.getNom() + ", prnom:" + bAnniversaire.getPrenom());
            	}
        	}
        	
        	// Lancement de l'ecran de consultation/modification d'vnement correspondant aux informations de l'lment de la liste
    		runCreationEventActivity(bAnniversaire.getId());
    	}
    }
    
    /**
     * Lancement de l'activit permettant de consulter/renseigner les informations d'un vnement
     * 
     * Si id vaut -1, alors cration d'un nouvel vnement, sinon, consultation/modification
     * 
     * @param id
     */
    protected void runCreationEventActivity (int id)
    {
		Intent intentEventCreer=new Intent (MainActivity.this, CreationEventActivity.class);
		
		if (id!=-1)
		{
			intentEventCreer.putExtra("id", id);
		}
		startActivity(intentEventCreer);    	
    }
    
    protected void runListEventActivity ()
    {
		Intent intentEventCreer=new Intent (MainActivity.this, ListEventActivity.class);
		
		startActivity(intentEventCreer);    	
    }
    
    /**
     * Classe permettant le chargement dans un thread (AsyncTask) des vnements puis prparation de la liste de l'activit
     * avec ces informations.
     * @author supertos
     *
     */
    class ThreadChargerListAnniversaires extends AsyncTask<Void, Long, Void> {

		ProgressDialog dialogWait;

		@Override
		protected void onPreExecute() {
			textViewListAnniversaire.setVisibility(View.INVISIBLE);
			textViewPasAnniv.setVisibility(View.INVISIBLE);
			textViewProchainAnniv.setVisibility(View.INVISIBLE);
			dialogWait = ProgressDialog.show(MainActivity.this, getString (R.string.info_recup_liste), getString (R.string.info_patientez), true);
		}

		@Override
		protected void onProgressUpdate(Long... values){
			// Rien  faire...
		}

		@Override
		protected Void doInBackground(Void... arg0) {

			// Connexion  la base
			sqLiteAnniv = new SqLiteAnniversaire();
			idSqLite=sqLiteAnniv.openDB(baseContext);
			
			// Rcupration d'un DAO sur table Anniversaires
			DaoAnniversaires daoAnniversaires=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
			
			Calendar c = Calendar.getInstance(); // date du jour
			
			// Rcupration des anniversaires/vnements  la date du jour
			listAnniversaires=daoAnniversaires.listAnniversaires(c.get(Calendar.YEAR), c.get(Calendar.MONTH)+1, c.get(Calendar.DAY_OF_MONTH));
			
			// Si aucun vnements
			if (listAnniversaires.isEmpty())
			{
				// Recherche des prochains anniversaires
				listFutursAnniversaires=daoAnniversaires.listFutursAnniversaires();
			}
			// Fermeture accs base
			if (sqLiteAnniv!=null && idSqLite!=null)
				sqLiteAnniv.closeDB (idSqLite);
				
			return null;
		}

		@Override
		protected void onPostExecute(Void result) {
			// Le chargement des donnes tant maitenant termin, nous pouvons prparer la liste avec ces informations.
			if (listAnniversaires.isEmpty())
			{
				// Pas d'anniversaire aujourd'hui !
				textViewListAnniversaire.setVisibility(View.GONE);
				listView.setVisibility(View.GONE);
				textViewPasAnniv.setVisibility(View.VISIBLE);
				textViewProchainAnniv.setVisibility (View.VISIBLE);
				if (!listFutursAnniversaires.isEmpty())
				{
					// Mettons la liste des personnes des prochains anniversaires...
					items=new String[listFutursAnniversaires.size()];
					int i=0;
					for (BeanAnniversaires element:listFutursAnniversaires)
					{
						Resources res = getResources();
						Integer nbJours=element.getNbJoursAvantAnniversaire();
						String jours=res.getQuantityString(R.plurals.texteJour, nbJours, nbJours); ;
						items[i++]=element.getPrenom() + " " + element.getNom() + " (" + jours + ")";
					}
					
					monAdaptater = new ArrayAdapter<String> (baseContext, android.R.layout.simple_list_item_1, items);
								
					listView.setAdapter(monAdaptater);
					
					// Affichons la liste !
					listView.setVisibility(View.VISIBLE);
				}	
				// else ... il n'y a pas d'enregistrement dans cette table !
			}
			else
			{
				// Mettons la liste des personnes dont c'est l'anniversaire...
				items=new String[listAnniversaires.size()];
				int i=0;
				for (BeanAnniversaires element:listAnniversaires)
				{
					Resources res = getResources();
					String ans=(element.getAge()>1 ? res.getString(R.string.an_pluriel):res.getString(R.string.an_singulier));
					items[i++]=element.getPrenom() + " " + element.getNom() + (element.getAnneeConnue() ? "("+element.getAge()+" "+ans+")":"");
					if (LogAnniv.isDebug())
			        {
			        	LogAnniv.i (CLASSE, "Element en cours:" + element.getPrenom() + " " + element.getNom());
			        }
				}
				
				monAdaptater = new ArrayAdapter<String> (baseContext, android.R.layout.simple_list_item_1, items);
							
				listView.setAdapter(monAdaptater);
				
				// Affichons les objets !
				listView.setVisibility(View.VISIBLE);
				textViewListAnniversaire.setVisibility(View.VISIBLE);
				textViewProchainAnniv.setVisibility (View.GONE);
				
				// Gestion du pluriel pour anniversaire
				Resources resources = getResources();
				
				String strAnniversaire = resources.getQuantityString(R.plurals.texteAnniversaire, listAnniversaires.size(), (Object[]) null);
				textViewListAnniversaire.setText(strAnniversaire);
				
				textViewPasAnniv.setVisibility(View.GONE);
			}
			
			// On peut fermer la dialogue permettant de faire patienter l'utilisateur...
			dialogWait.dismiss();

		}
	}
}
