package fr.free.supertos.anniversaire;

import android.app.Activity;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TimePicker;
import fr.free.supertos.anniversaire.bean.BeanParametres;
import fr.free.supertos.anniversaire.dao.DaoParametres;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;
import fr.free.supertos.anniversaire.service.ServiceAutoRun;

/**
 * Paramtrage de l'alarme
 * @author supertos
 *
 */
public class ParamActivityAlarm extends Activity implements OnClickListener{

	private final static String CLASSE = ParamActivityAlarm.class.getName();
	
	SqLiteAnniversaire sqLiteAnniv;
	SQLiteDatabase idSqLite;
	
	TimePicker timePickerAlarm;
	TimePicker timePickerRappel;
	
	Button buttonValider;
	Button buttonAnnuler;

	private boolean restaurationFaite=false;
	
	private BeanParametres bParametres;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_parametres_alarm);
		
		timePickerAlarm=(TimePicker)findViewById(R.id.timePickerAlarm);

		buttonValider=(Button)findViewById(R.id.buttonValider);
		buttonValider.setOnClickListener((OnClickListener)this); 
		buttonAnnuler=(Button)findViewById(R.id.buttonAnnuler);
		buttonAnnuler.setOnClickListener((OnClickListener)this);
		
		timePickerAlarm.setIs24HourView(true);
		
		restaurerInstance(savedInstanceState);
	}

	private void restaurerInstance(Bundle savedInstanceState) {
		
		if (savedInstanceState!=null)
		{	
			bParametres = new BeanParametres();
			
			bParametres.setHeureAlarme(savedInstanceState.getInt("heure"));
			bParametres.setMinuteAlarme(savedInstanceState.getInt("minute"));
		}
		else
		{
			sqLiteAnniv = new SqLiteAnniversaire();
			idSqLite=sqLiteAnniv.openDB(this);
			
			DaoParametres daoParametres=(DaoParametres)DaoFactory.daoParametres(idSqLite);
			bParametres = daoParametres.getParametres();
			
			sqLiteAnniv.closeDB (idSqLite);		
		}
	}

	@Override
	protected void onSaveInstanceState(Bundle saveInstanceState) {

		super.onSaveInstanceState(saveInstanceState);

		saveInstanceState.putInt ("heure", timePickerAlarm.getCurrentHour());
	    saveInstanceState.putInt ("minute", timePickerAlarm.getCurrentMinute());
	}
	
	public void onResume()
	{
		super.onResume();
		
		if (LogAnniv.isDebug())
		{
			LogAnniv.d (CLASSE, "Les parametres sont:"+bParametres.getDelaiRepeterAlarme()+" "+ bParametres.getHeureAlarme() + " "+bParametres.getMinuteAlarme());
		}

		timePickerAlarm.setCurrentHour(bParametres.getHeureAlarme());
		timePickerAlarm.setCurrentMinute(bParametres.getMinuteAlarme());
	}
	
	public void onPause()
	{		
		super.onPause();
		
		bParametres.setHeureAlarme(timePickerAlarm.getCurrentHour());
		bParametres.setMinuteAlarme(timePickerAlarm.getCurrentMinute());
	}
	
	public void onDestroy()
	{	
		super.onDestroy();
	}
	
	public void onClick(View view){
    	switch (view.getId())
    	{	    	
	    	case R.id.buttonValider:
	        	SqLiteAnniversaire sqLiteAnniv = new SqLiteAnniversaire();
	    		SQLiteDatabase idSqLite=sqLiteAnniv.openDB(this);
	    		
	    		DaoParametres daoParametre=(DaoParametres)DaoFactory.daoParametres(idSqLite);
	    		BeanParametres bParametres=new BeanParametres();
	    		bParametres.setHeureAlarme(timePickerAlarm.getCurrentHour());
	    		bParametres.setMinuteAlarme(timePickerAlarm.getCurrentMinute());
	    		daoParametre.ecrireParametresAlarme(bParametres);
	    		
	    		sqLiteAnniv.closeDB (idSqLite);
	    		
	    		// Relance du service	    		
	    	    Intent intentServiceAutoRun = new Intent(ParamActivityAlarm.this, ServiceAutoRun.class);
	    	    intentServiceAutoRun.putExtra("CHANGE", true);
	    		startService(intentServiceAutoRun);

	        	finish();
	    	break;

	    	case R.id.buttonAnnuler:
	    		finish();
	    	break;  	
    	}
    }

}
