package fr.free.supertos.anniversaire;

import android.app.Activity;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.Button;
import android.widget.TimePicker;
import fr.free.supertos.anniversaire.bean.BeanParametres;
import fr.free.supertos.anniversaire.dao.DaoParametres;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.service.ServiceAutoRun;

public class ParamActivityRappel extends Activity implements OnClickListener {

	private final static String CLASSE = ParamActivityRappel.class.getName();
	
	SqLiteAnniversaire sqLiteAnniv;
	SQLiteDatabase idSqLite;

	TimePicker timePickerRappel;
	
	Button buttonValider;
	Button buttonAnnuler;

	private BeanParametres bParametres;
	
	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_parametres_rappel);
		
		timePickerRappel=(TimePicker)findViewById(R.id.timePickerRappel);

		buttonValider=(Button)findViewById(R.id.buttonValider);
		buttonValider.setOnClickListener((OnClickListener)this); 
		buttonAnnuler=(Button)findViewById(R.id.buttonAnnuler);
		buttonAnnuler.setOnClickListener((OnClickListener)this); 
		
		timePickerRappel.setIs24HourView(true);
		
		restaurerInstance(savedInstanceState);
	}

	private void restaurerInstance(Bundle savedInstanceState) {
		
		// Rcupration des donnes sauves dans le bundle
		if (savedInstanceState!=null)
		{			
			bParametres = new BeanParametres();
			
			bParametres.setDelaiRepeterAlarme(savedInstanceState.getInt("heure")*60 + savedInstanceState.getInt("minute"));
		}
		else
		{
			// Sinon, rcupration des donnes en base
			sqLiteAnniv = new SqLiteAnniversaire();
			idSqLite=sqLiteAnniv.openDB(this);
			
			DaoParametres daoParametres=(DaoParametres)DaoFactory.daoParametres(idSqLite);
			bParametres=daoParametres.getParametres();

			sqLiteAnniv.closeDB (idSqLite);			
		}
	}
	
	@Override
	protected void onSaveInstanceState(Bundle saveInstanceState) {

		super.onSaveInstanceState(saveInstanceState);

		// Destruction de l'activit, sauvegarde des donnes pour reprise si prochaine restauration.
		saveInstanceState.putInt ("heure", timePickerRappel.getCurrentHour());
	    saveInstanceState.putInt ("minute", timePickerRappel.getCurrentMinute());
	}
	
	public void onResume()
	{
		super.onResume();
			
		if (bParametres.getDelaiRepeterAlarme() != 0)
		{
			float f = bParametres.getDelaiRepeterAlarme()/60;
			int i = (int) f;
			int j = bParametres.getDelaiRepeterAlarme()-i*60;
			timePickerRappel.setCurrentHour(i);
			timePickerRappel.setCurrentMinute(j);
		}
		else
		{
			timePickerRappel.setCurrentHour(0);
			timePickerRappel.setCurrentMinute(0);
		}
	}
	
	public void onClick(View view){
    	switch (view.getId())
    	{	    	
	    	case R.id.buttonValider:
	        	SqLiteAnniversaire sqLiteAnniv = new SqLiteAnniversaire();
	    		SQLiteDatabase idSqLite=sqLiteAnniv.openDB(this);
	    		
	    		DaoParametres daoParametre=(DaoParametres)DaoFactory.daoParametres(idSqLite);
	    		BeanParametres bParametres=new BeanParametres();
	    		bParametres.setDelaiRepeterAlarme(timePickerRappel.getCurrentHour()*60 + timePickerRappel.getCurrentMinute());
	    		daoParametre.ecrireParametresRappel(bParametres);
	    		
	    		sqLiteAnniv.closeDB (idSqLite);
	    		
	    		// Relance du service	    		
	    	    Intent intentServiceAutoRun = new Intent(ParamActivityRappel.this, ServiceAutoRun.class);
	    	    intentServiceAutoRun.putExtra("CHANGE", true);
	    		startService(intentServiceAutoRun);

	        	finish();
	    	break;

	    	case R.id.buttonAnnuler:
	    		finish();
	    	break;  	
    	}
    }
	
	public void onPause()
	{		
		super.onPause();
		// En pause, mmorisation des donnes saisies pour une activit non dtruite.
		bParametres.setDelaiRepeterAlarme(timePickerRappel.getCurrentHour()*60 + timePickerRappel.getCurrentMinute());
	}
	
	public void onDestroy()
	{

		super.onDestroy();
	}

}
