package fr.free.supertos.anniversaire.dao;

import java.io.IOException;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;
import java.util.Set;
import java.util.TreeMap;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xmlpull.v1.XmlSerializer;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import fr.free.supertos.anniversaire.bean.BeanAnniversaires;
import fr.free.supertos.anniversaire.constantes.ConstantesAnniversaires;
import fr.free.supertos.anniversaire.log.LogAnniv;

/**
 * DAO pour la table anniversaires
 * 
 * @author supertos
 *
 */
public class DaoAnniversaires extends DaoStandard {
	
	private final static String CLASSE = DaoAnniversaires.class.getName();
	
	// Constructeur 
	public DaoAnniversaires (SQLiteDatabase idSQLiteDB) {
		super(idSQLiteDB);
	}
	
	public DaoAnniversaires (XmlSerializer serializer) {
		super(serializer);
	}
	
	/**
	 * Retourne la liste des bean anniversaires rpondant  une date donne en paramtre
	 * 
	 * @param iAnnee
	 * @param iMois
	 * @param iJour
	 * @return List<BeanAnniversaires>
	 */
	public List<BeanAnniversaires> listAnniversaires (int iAnnee, int iMois, int iJour)
	{
		String mois=String.valueOf(iMois);
		String jour=String.valueOf(iJour);
		String requete="mois=? and jour=?";
		String [] dateAnniv={mois, jour};
		
		return traitementListAnniversaires (iAnnee, requete, dateAnniv);
	}

	/**
	 * Retourne la liste des bean anniversaires rpondant  la date courante
	 * 
	 * @return List<BeanAnniversaires>
	 */
	public List<BeanAnniversaires> listAnniversaires ()
	{
		if (idSQLiteDB!=null)
		{
			String requete=null;
			String [] dateAnniv=null;
			
			Calendar date= Calendar.getInstance();
			int iAnnee=date.get(Calendar.YEAR);
			return traitementListAnniversaires (iAnnee, requete, dateAnniv);
		}
		
		return null;
	}
	
	public List<BeanAnniversaires> listAnniversaires (Document dom)
	{
		if (idSQLiteDB!=null)
		{
			String requete=null;
			String [] dateAnniv=null;
			
			Calendar date= Calendar.getInstance();
			int iAnnee=date.get(Calendar.YEAR);
			return traitementListAnniversaires (iAnnee, requete, dateAnniv);
		}
		
		if (serializer!=null)
		{
			return traitementListAnniversaires(dom);
		}
		return null;
	}

	private List<BeanAnniversaires> traitementListAnniversaires(Document doc) {
		
		// Rcupration des records
		LogAnniv.d(CLASSE,  "root element:" +  doc.getDocumentElement().getNodeName());
		
		// Rcupration des records
		NodeList listOfEvents = doc.getElementsByTagName("record");
        int totalEvens = listOfEvents.getLength();

        LogAnniv.d(CLASSE,  "Nb event:" +  totalEvens);
        
        List<BeanAnniversaires> listBAnniversaires=new ArrayList<BeanAnniversaires>();
        
        for(int s=0; s<listOfEvents.getLength() ; s++){


            Node firstEventNode = listOfEvents.item(s);
            if(firstEventNode.getNodeType() == Node.ELEMENT_NODE){

            	BeanAnniversaires bRecord=new BeanAnniversaires();
            	
                Element firstEventElement = (Element)firstEventNode;

                //-------
                NodeList nomList = firstEventElement.getElementsByTagName("nom");
                Element nomElement = (Element)nomList.item(0);
                NodeList textNomList = ((Node) nomElement).getChildNodes();
                
                // Les informations nom et prenom provenant d'une application autre, ils peuvent tre null
                // Il faut donc traper NullPointerException
                String nom;
                try
                {
                	nom=((Node)textNomList.item(0)).getNodeValue();
                	nom = nom.trim();
                }catch (NullPointerException e) {
                	nom = null;
                }
//                LogAnniv.d(CLASSE, "Nom: " + nom);
               	bRecord.setNom(nom);
                
                //-------
                NodeList prenomList = firstEventElement.getElementsByTagName("prenom");
                Element prenomElement = (Element)prenomList.item(0);

                NodeList textPrenomList = ((Node) prenomElement).getChildNodes();
                String prenom;
                try
                {
                	prenom=((Node)textPrenomList.item(0)).getNodeValue();
                	prenom = prenom.trim();
                }catch (NullPointerException e) {
                	prenom = null;
                }
//                LogAnniv.d(CLASSE, "Prnom: " + prenom);
                bRecord.setPrenom(prenom);
                
                //idContact
                NodeList idContactList = firstEventElement.getElementsByTagName("idContact");
                Element idContactElement = (Element)idContactList.item(0);
                
                NodeList textIdContactList = ((Node) idContactElement).getChildNodes();
//                LogAnniv.d(CLASSE, "ID Contact: " + ((Node)textIdContactList.item(0)).getNodeValue().trim());
                bRecord.setIdContact(Long.valueOf(((Node)textIdContactList.item(0)).getNodeValue().trim()));
                
                //anneeconnue
                NodeList AnneeConnueList = firstEventElement.getElementsByTagName("anneeconnue");
                Element AnneeConnueElement = (Element)AnneeConnueList.item(0);
                
                NodeList textAnneeConnueList = ((Node) AnneeConnueElement).getChildNodes();
//                LogAnniv.d(CLASSE, "annee connue: " + ((Node)textAnneeConnueList.item(0)).getNodeValue().trim());
                bRecord.setAnneeConnue(Integer.valueOf(((Node)textAnneeConnueList.item(0)).getNodeValue().trim()));

                //annee
                NodeList anneeList = firstEventElement.getElementsByTagName("annee");
                Element anneeElement = (Element)anneeList.item(0);
                
                NodeList textAnneeList = ((Node) anneeElement).getChildNodes();
//                LogAnniv.d(CLASSE, "annee: " + ((Node)textAnneeList.item(0)).getNodeValue().trim());
                bRecord.setAnnee(Integer.valueOf(((Node)textAnneeList.item(0)).getNodeValue().trim()));
                
                //mois
                NodeList moisList = firstEventElement.getElementsByTagName("mois");
                Element moisElement = (Element)moisList.item(0);
                
                NodeList textMoisList = ((Node) moisElement).getChildNodes();
//                LogAnniv.d(CLASSE, "mois: " + ((Node)textMoisList.item(0)).getNodeValue().trim());
                bRecord.setMois(Integer.valueOf(((Node)textMoisList.item(0)).getNodeValue().trim()));

                //jour
                NodeList jourList = firstEventElement.getElementsByTagName("jour");
                Element jourElement = (Element)jourList.item(0);
                
                NodeList textJourList = ((Node) jourElement).getChildNodes();
//                LogAnniv.d(CLASSE, "jour: " + ((Node)textJourList.item(0)).getNodeValue().trim());
                bRecord.setJour(Integer.valueOf(((Node)textJourList.item(0)).getNodeValue().trim()));

                //autreeven
                NodeList autreEventList = firstEventElement.getElementsByTagName("autreeven");
                Element autreEventElement = (Element)autreEventList.item(0);
                
                NodeList textautreEventList = ((Node) autreEventElement).getChildNodes();
//                LogAnniv.d(CLASSE, "autre evenement: " + ((Node)textIdContactList.item(0)).getNodeValue().trim());
                bRecord.setAutreEven(Integer.valueOf(((Node)textautreEventList.item(0)).getNodeValue().trim()));
                
                Node item = listOfEvents.item(s);
                String id=item.getAttributes().getNamedItem("id").getNodeValue();
        		bRecord.setId(Integer.valueOf(id));
//        		LogAnniv.d(CLASSE, "id: " + id);
        		
        		if (bRecord.getAnneeConnue())
        		{
        			Calendar c=Calendar.getInstance();
        			bRecord.setAge(c.get(Calendar.YEAR)-bRecord.getAnnee());
        			if (bRecord.getAge() < 0)
        			{
        				bRecord.setAge(-1);
        			}
        		}
        		else
        		{
        			bRecord.setAge(-1);
        		}
        		
        		bRecord.setNomPrenom(bRecord.getNom().toUpperCase()+bRecord.getPrenom().toUpperCase());
        		
        		if (LogAnniv.isDebug())
        		{
        			LogAnniv.d (CLASSE, "Bean ajout:" + bRecord.getId() + " " + bRecord.getNom() + " " + bRecord.getPrenom());
        		}
        		
        		listBAnniversaires.add(bRecord);

            }
        }
		return listBAnniversaires;
	}

	public List<BeanAnniversaires> traitementListAnniversaires (int iAnnee, String requete, String[] valeur)
	{
        String [] columns=new String[]{"id","idContact","nomprenom","nom","prenom", "annee", "mois", "jour","anneeconnue","autreeven"};
        
        List<BeanAnniversaires> listBAnniversaires=new ArrayList<BeanAnniversaires>();
        
        String orderBy = "nomprenom ASC";
        
		Cursor cursor = idSQLiteDB.query("anniversaires", columns, requete,valeur,null,null,orderBy);
        // Affichage des enregistrements
        if (cursor.getCount() > 0) // Du moins, s'il y en a !
        {               
        	cursor.moveToFirst(); // On se positionne sur le premier
            do {
            	if (LogAnniv.isDebug())
            	{
            		LogAnniv.d (CLASSE, "Enregistrement n" + cursor.getInt(cursor.getColumnIndex("id")) + ": nom=" + cursor.getString(cursor.getColumnIndex("nom")) + ", prenom=" + cursor.getString(cursor.getColumnIndex("prenom")));
            	}
        		BeanAnniversaires bRecord=new BeanAnniversaires();
        		
        		bRecord.setId(cursor.getInt(cursor.getColumnIndex("id")));
        		bRecord.setIdContact(cursor.getInt(cursor.getColumnIndex("idContact")));
        		bRecord.setNomPrenom(cursor.getString(cursor.getColumnIndex("nom"))+cursor.getString(cursor.getColumnIndex("prenom")).toUpperCase());
        		bRecord.setNom(cursor.getString(cursor.getColumnIndex("nom")));
        		bRecord.setPrenom(cursor.getString(cursor.getColumnIndex("prenom")));
        		bRecord.setAnneeConnue(cursor.getInt(cursor.getColumnIndex("anneeconnue")));
        		bRecord.setAutreEven(cursor.getInt(cursor.getColumnIndex("autreeven")));
        		bRecord.setAnnee(cursor.getInt(cursor.getColumnIndex("annee")));
        		bRecord.setMois (cursor.getInt(cursor.getColumnIndex("mois")));
        		bRecord.setJour (cursor.getInt(cursor.getColumnIndex("jour")));
            	
        		int anneeconnue=cursor.getInt(cursor.getColumnIndex("anneeconnue"));
        		if (iAnnee>=cursor.getInt(cursor.getColumnIndex("annee")) && anneeconnue==ConstantesAnniversaires.ANNEECONNUE)
        		{
        			bRecord.setAge(iAnnee-cursor.getInt(cursor.getColumnIndex("annee")));
        		}
        		else
        		{
        			bRecord.setAge(-1);
        		}
        		listBAnniversaires.add(bRecord);
            } while (cursor.moveToNext()); // Puis sur les suivants
            
            cursor.close(); // Fermeture du curseur
        }
        if (LogAnniv.isInformation())
        {
        	LogAnniv.i (CLASSE, "NB. records:" + cursor.getCount());
        }
        return listBAnniversaires;

	}
	public void ecrireAnniversaire (BeanAnniversaires bAnniversaire)
	{
	
		// Ecriture vers la base SQL
		if (idSQLiteDB!=null)
		{
			ContentValues cv = new ContentValues();
	
			cv.put("nomprenom", bAnniversaire.getNom().toUpperCase()+bAnniversaire.getPrenom().toUpperCase());
			cv.put("nom", bAnniversaire.getNom());
			cv.put("prenom", bAnniversaire.getPrenom());
			cv.put("annee", String.valueOf(bAnniversaire.getAnnee()));
			cv.put("mois", String.valueOf(bAnniversaire.getMois()));
			cv.put("jour", String.valueOf(bAnniversaire.getJour()));
			cv.put("idContact", bAnniversaire.getIdContact());
			if (bAnniversaire.getAnneeConnue())
			{
				cv.put("anneeconnue", ConstantesAnniversaires.ANNEECONNUE);
			}
			else
			{
				cv.put("anneeconnue", ConstantesAnniversaires.ANNEEINCONNUE);
			}
			cv.put("autreeven", String.valueOf(bAnniversaire.getIntAutreEven()));		
			if (bAnniversaire.getId()==null)
			{
				idSQLiteDB.insert("anniversaires", null, cv);
			}
			else
			{
				String where[]={String.valueOf(bAnniversaire.getId())};
				if (idSQLiteDB.update("anniversaires", cv, "id=?", where)==0)
				{
					// Il n'existe pas...
					idSQLiteDB.insert("anniversaires", null, cv);
				}
			}
		}
		
		// Ecriture vers un fichier XML pralablement prpar
		if (serializer!=null)
		{
			try {
				serializer.startTag(null, "record");
				
					serializer.attribute(null,"id",String.valueOf(bAnniversaire.getId()));
					serializer.startTag(null, "nom");
					serializer.text(bAnniversaire.getNom());
					serializer.endTag(null,"nom");
	
					serializer.startTag(null, "prenom");
					serializer.text(bAnniversaire.getPrenom());
					serializer.endTag(null,"prenom");
	
					serializer.startTag(null, "annee");
					serializer.text(String.valueOf(bAnniversaire.getAnnee()));
					serializer.endTag(null,"annee");
	
					serializer.startTag(null, "mois");
					serializer.text(String.valueOf(bAnniversaire.getMois()));
					serializer.endTag(null,"mois");
	
					serializer.startTag(null, "jour");
					serializer.text(String.valueOf(bAnniversaire.getJour()));
					serializer.endTag(null,"jour");
	
					serializer.startTag(null, "idContact");
					serializer.text(String.valueOf(bAnniversaire.getIdContact()));
					serializer.endTag(null,"idContact");
	
					serializer.startTag(null, "anneeconnue");
					if (bAnniversaire.getAnneeConnue())
					{
						serializer.text(String.valueOf(ConstantesAnniversaires.ANNEECONNUE));
					}
					else
					{
						serializer.text(String.valueOf(ConstantesAnniversaires.ANNEEINCONNUE));
					}
					serializer.endTag(null,"anneeconnue");
					
					serializer.startTag(null, "autreeven");
					serializer.text(String.valueOf(bAnniversaire.getIntAutreEven()));
					serializer.endTag(null,"autreeven");
				
				serializer.endTag(null,	"record");

			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalStateException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NullPointerException e) {
				LogAnniv.e (CLASSE, "NullPointerException:Certainement un bean avec un getter retournant un null:" + bAnniversaire.getId());
				e.printStackTrace();
			}
		}
	}

	public int supprimerAnniversaire (Integer id)
	{
		String sId=String.valueOf(id);
		
		String [] idAnniv={sId};
        
        LogAnniv.d(CLASSE, "---------- Suppression id="+id);
        return idSQLiteDB.delete("anniversaires", "id=?",idAnniv);
	}
	
	public BeanAnniversaires lireAnniversaire(Integer id) {
		
		String sId=String.valueOf(id);
		
		String [] idAnniv={sId};
		
        String [] columns=new String[]{"id","idContact","nomprenom", "nom","prenom", "annee","anneeconnue","autreeven","mois","jour"};
        
        BeanAnniversaires bAnniversaire=new BeanAnniversaires();
        
        LogAnniv.d(CLASSE, "---------- id="+id);
        Cursor cursor = idSQLiteDB.query("anniversaires", columns, "id=?",idAnniv,null,null,"1");
        // Affichage des enregistrements
        if (cursor.getCount() > 0) // Du moins, s'il y en a !
        {               
        	cursor.moveToFirst(); // On se positionne sur l'enregistrement
        	if (LogAnniv.isDebug())
        	{
        		LogAnniv.d (CLASSE, "Enregistrement n" + cursor.getInt(cursor.getColumnIndex("id")) + ": nom=" + cursor.getString(cursor.getColumnIndex("nom")) + ", prenom=" + cursor.getString(cursor.getColumnIndex("prenom")) + ", annee connue:"+cursor.getInt(cursor.getColumnIndex("anneeconnue")));
        	}
    		
        	bAnniversaire.setId(cursor.getInt(cursor.getColumnIndex("id")));
        	bAnniversaire.setIdContact(cursor.getInt(cursor.getColumnIndex("idContact")));
        	bAnniversaire.setNomPrenom(cursor.getString(cursor.getColumnIndex("nom")).toUpperCase()+cursor.getString(cursor.getColumnIndex("prenom")).toUpperCase());
        	bAnniversaire.setNom(cursor.getString(cursor.getColumnIndex("nom")));
        	bAnniversaire.setPrenom(cursor.getString(cursor.getColumnIndex("prenom")));
        	bAnniversaire.setAnneeConnue(cursor.getInt(cursor.getColumnIndex("anneeconnue")));
        	bAnniversaire.setAutreEven(cursor.getInt(cursor.getColumnIndex("autreeven")));
        	bAnniversaire.setAnnee (cursor.getInt(cursor.getColumnIndex("annee")));
        	bAnniversaire.setMois (cursor.getInt(cursor.getColumnIndex("mois")));
        	bAnniversaire.setJour (cursor.getInt(cursor.getColumnIndex("jour")));
        	
    		int anneeconnue=cursor.getInt(cursor.getColumnIndex("anneeconnue"));
    		final Calendar date=Calendar.getInstance();
    		int iAnnee=date.get (Calendar.YEAR);
    		if (iAnnee>=cursor.getInt(cursor.getColumnIndex("annee")) && anneeconnue==ConstantesAnniversaires.ANNEECONNUE)
    		{
    			bAnniversaire.setAge(iAnnee-cursor.getInt(cursor.getColumnIndex("annee")));
    		}
    		else
    		{
    			bAnniversaire.setAge(-1);
    		}
            
            cursor.close(); // Fermeture du curseur
        }
        else
        {
        	return null;
        }
        if (LogAnniv.isInformation())
        {
        	LogAnniv.i (CLASSE, "NB. records:" + cursor.getCount());
        }
        return bAnniversaire;	}

	public List<BeanAnniversaires> listFutursAnniversaires()
	{
		List<BeanAnniversaires> listAnniversaires = this.listAnniversaires ();
				
		TreeMap<Long, BeanAnniversaires> treeMap = new TreeMap<Long, BeanAnniversaires>(); // Liste tri sur clef
		int x=0; // Squenceur
		
		for (BeanAnniversaires element:listAnniversaires)
		{
			
			
			// Pour effectuer le tri, nous allons utiliser le nb. de jours avant l'anniversaire + un squenceur 
			// afin d'viter des doublons qui ne sont pas autoriss dans la collection TreeMap
						
			Long sequenceur = (long) (element.getNbJoursAvantAnniversaire() * 1000 + x++);

			if (LogAnniv.isDebug())
	        {
	        	LogAnniv.i (CLASSE, "Date et nb jours avant anniversaire:" + " " + element.getJour() + "/" + element.getMois() + " = " + element.getNbJoursAvantAnniversaire());
	        }

			treeMap.put(sequenceur, element);

			if (LogAnniv.isDebug())
	        {
	        	LogAnniv.i (CLASSE, "Prochaine valeur du squenceur:" + sequenceur);
	        }
		}
		
		// Maintenant, prparation d'une liste trie pour retour  l'appelant
		Set<Long> cles = treeMap.keySet();
		Iterator<Long> iterator = cles.iterator();
		List<BeanAnniversaires> listFutursBAnniversaires=new ArrayList<BeanAnniversaires>();
		
		while (iterator.hasNext()) {
			listFutursBAnniversaires.add(treeMap.get(iterator.next()));
		}
		
        return listFutursBAnniversaires;
	}
	
	public void toutEffacer()
	{
		idSQLiteDB.delete("anniversaires", null,null);
	}
}
