package fr.free.supertos.anniversaire.dao;

import java.io.IOError;
import java.io.IOException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xmlpull.v1.XmlSerializer;

import android.content.ContentValues;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import fr.free.supertos.anniversaire.bean.BeanParametres;
import fr.free.supertos.anniversaire.log.LogAnniv;

public class DaoParametres extends DaoStandard {
	
	private final static String CLASSE = DaoParametres.class.getName();
	
	public DaoParametres(SQLiteDatabase idSqLite) {
		super(idSqLite);
	}
	
	public DaoParametres(XmlSerializer serializer) {
		super(serializer);	}
	
	public BeanParametres getParametres (Document doc)
	{
		BeanParametres bParametres=new BeanParametres();
		
		// Rcupration des records
		LogAnniv.d(CLASSE,  "root element:" +  doc.getDocumentElement().getNodeName());
		
		// Rcupration des records
		NodeList listOfParamters = doc.getElementsByTagName("record");
        int totalParam = listOfParamters.getLength();

        LogAnniv.d(CLASSE,  "Nb parametres:" +  totalParam);
        
        if (totalParam == 0)
        {
        	// Bizarre, il n'y a rien...
        	// Cration d'un bean par dfaut !
        	bParametres.setDelaiRepeterAlarme(180);
        	bParametres.setHeureAlarme(8);
        	bParametres.setMinuteAlarme(30); 
        	bParametres.setVibration(1);
        	bParametres.setSon(1);        	
        }else if (totalParam == 1)
        {
        	// Cas normal
            Node firstParametreNode = listOfParamters.item(0);
            if(firstParametreNode.getNodeType() == Node.ELEMENT_NODE){

                Element firstParametreElement = (Element)firstParametreNode;

                //-------
                NodeList delaiRepeterAlarmeList = firstParametreElement.getElementsByTagName("delaiRepeterAlarme");
                Element delaiRepeterAlarmeElement = (Element)delaiRepeterAlarmeList.item(0);
                NodeList textDelaiRepeterAlarmeList = ((Node) delaiRepeterAlarmeElement).getChildNodes();
                
                Integer delaiRepeterAlarme;
               	delaiRepeterAlarme=Integer.valueOf(((Node)textDelaiRepeterAlarmeList.item(0)).getNodeValue());
                bParametres.setDelaiRepeterAlarme(delaiRepeterAlarme);
                
                //-------
                NodeList heureAlarmeList = firstParametreElement.getElementsByTagName("heureAlarme");
                Element heureAlarmeElement = (Element)heureAlarmeList.item(0);

                NodeList textHeureAlarmeList = ((Node) heureAlarmeElement).getChildNodes();
                Integer heure;
               	heure=Integer.valueOf(((Node)textHeureAlarmeList.item(0)).getNodeValue());
                bParametres.setHeureAlarme(heure);
                
                //---------
                NodeList minuteAlarmeList = firstParametreElement.getElementsByTagName("minuteAlarme");
                Element minuteAlarmeElement = (Element)minuteAlarmeList.item(0);
                
                NodeList textMinuteAlarmeList = ((Node) minuteAlarmeElement).getChildNodes();
                Integer minute;
               	minute=Integer.valueOf(((Node)textMinuteAlarmeList.item(0)).getNodeValue());
                bParametres.setMinuteAlarme(minute);

                //---------
                NodeList vibrationList = firstParametreElement.getElementsByTagName("vibration");
                Element vibrationElement = (Element)vibrationList.item(0);
                
                NodeList textVibrationList = ((Node) vibrationElement).getChildNodes();
                Integer vibration;
                vibration=Integer.valueOf(((Node)textVibrationList.item(0)).getNodeValue());
                bParametres.setVibration(vibration);

                //---------
                NodeList sonList = firstParametreElement.getElementsByTagName("son");
                Element sonElement = (Element)sonList.item(0);
                
                NodeList textSonList = ((Node) sonElement).getChildNodes();
                Integer son;
                son=Integer.valueOf(((Node)textSonList.item(0)).getNodeValue());
                bParametres.setSon(son);
            }

        }else
        {
        	throw  new IOError (new Throwable ("Il ne peut y avoir qu'un enregistrement dans paramtres. Nb trouv:"+totalParam));
        }
		
		return bParametres;
	}
	
	public BeanParametres getParametres () {	
        String [] columns=new String[]{"id,delaiRepeterAlarme","heureAlarme","minuteAlarme,vibration,son"};
        String [] where=new String[]{"0"};
        
        BeanParametres bParametres=new BeanParametres();
        
        Cursor cursor = idSQLiteDB.query("parametres", columns, "id>=?",where,null,null,null);
        // Affichage des enregistrements
        Integer nbRecord = cursor.getCount();
        if (nbRecord > 0) // Du moins, s'il y en a !
        {               
        	cursor.moveToFirst(); // On se positionne sur le premier
            
        	if (LogAnniv.isDebug())
        	{
        		LogAnniv.d (CLASSE, "Enregistrement n" + cursor.getInt(cursor.getColumnIndex("id")) + ": delaiRepeterAlarme=" + cursor.getString(cursor.getColumnIndex("delaiRepeterAlarme")) + ", heureAlarme=" + cursor.getString(cursor.getColumnIndex("heureAlarme"))+", minuteAlarme=" + cursor.getString(cursor.getColumnIndex("minuteAlarme")));
        	}        		
        	bParametres.setDelaiRepeterAlarme(cursor.getInt(cursor.getColumnIndex("delaiRepeterAlarme")));
        	bParametres.setHeureAlarme(cursor.getInt(cursor.getColumnIndex("heureAlarme")));
        	bParametres.setMinuteAlarme(cursor.getInt(cursor.getColumnIndex("minuteAlarme")));
        	bParametres.setVibration(cursor.getInt(cursor.getColumnIndex("vibration")));
        	bParametres.setSon(cursor.getInt(cursor.getColumnIndex("son")));
            
            cursor.close(); // Fermeture du curseur
        }
        else
        {
        	// problme, il n'y en a pas !
        	// Cration d'un bean par dfaut !
        	bParametres.setDelaiRepeterAlarme(180);
        	bParametres.setHeureAlarme(8);
        	bParametres.setMinuteAlarme(30); 
        	bParametres.setVibration(1);
        	bParametres.setSon(1);
        	
        	LogAnniv.w (CLASSE, "Table parametres: record anormalement absent, cration d'un par dfaut...");
        	
    		ContentValues cv = new ContentValues();
    		// Cration de paramtres par dfaut
    		cv.put ("delaiRepeterAlarme", "180");
    		cv.put ("heureAlarme", "8");
    		cv.put ("minuteAlarme", "30");
    		cv.put ("vibration", "1");
    		cv.put ("son", "1");
    		
    		idSQLiteDB.insert("parametres", null, cv);
        }
        
        if (nbRecord > 1)
        	LogAnniv.w (CLASSE, "Table parametres: nb. records anormalement suprieur  1, le premier est pris." + nbRecord);
        return bParametres;
	}

	public void ecrireParametresAlarme(BeanParametres bParametres) {
		ContentValues cv = new ContentValues();

		cv.put("heureAlarme", bParametres.getHeureAlarme());
		cv.put("minuteAlarme", bParametres.getMinuteAlarme());

		idSQLiteDB.update("parametres", cv, null, null);
	}
	
	public void ecrireParametresRappel(BeanParametres bParametres) {
		ContentValues cv = new ContentValues();

		cv.put("delaiRepeterAlarme", bParametres.getDelaiRepeterAlarme());
		
		idSQLiteDB.update("parametres", cv, null, null);
	}

	public void ecrireParametresNotification(BeanParametres bParametres) {
		ContentValues cv = new ContentValues();

		cv.put("vibration", bParametres.getVibration());
		cv.put("son", bParametres.getSon());

		idSQLiteDB.update("parametres", cv, null, null);
	}

	public void ecrireParametres(BeanParametres bParametres)
	{

		if (idSQLiteDB!=null)
		{
			String [] columns=new String[]{"id,delaiRepeterAlarme","heureAlarme","minuteAlarme,vibration,son"};
	        String [] where=new String[]{"0"};
	        
			Cursor cursor = idSQLiteDB.query("parametres", columns, "id>=?",where,null,null,null);
	        // Affichage des enregistrements
	        Integer nbRecord = cursor.getCount();
	        
			ContentValues cv = new ContentValues();
			// Cration de paramtres par dfaut
			cv.put ("delaiRepeterAlarme", String.valueOf(bParametres.getDelaiRepeterAlarme()));
			cv.put ("heureAlarme", String.valueOf(bParametres.getHeureAlarme()));
			cv.put ("minuteAlarme", String.valueOf(bParametres.getMinuteAlarme()));
			cv.put ("vibration", String.valueOf(bParametres.getVibration()));
			cv.put ("son", String.valueOf(bParametres.getSon()));
			
			LogAnniv.d ("DAO...","nb.param=" + nbRecord);
			
	        if (nbRecord > 0) // Du moins, s'il y en a !
	        {               
	        	idSQLiteDB.update("parametres", cv, null, null);
	        }
	        else
	        {
	        	idSQLiteDB.insert("parametres", null, cv);
	        }
		}
		
		if (serializer != null)
		{
			try {
				serializer.startTag(null, "record");
				
					serializer.startTag(null, "delaiRepeterAlarme");
					serializer.text(String.valueOf(bParametres.getDelaiRepeterAlarme()));
					serializer.endTag(null,"delaiRepeterAlarme");
	
					serializer.startTag(null, "heureAlarme");
					serializer.text(String.valueOf(bParametres.getHeureAlarme()));
					serializer.endTag(null,"heureAlarme");
	
					serializer.startTag(null, "minuteAlarme");
					serializer.text(String.valueOf(bParametres.getMinuteAlarme()));
					serializer.endTag(null,"minuteAlarme");
	
					serializer.startTag(null, "vibration");
					serializer.text(String.valueOf(bParametres.getVibration()));
					serializer.endTag(null,"vibration");
	
					serializer.startTag(null, "son");
					serializer.text(String.valueOf(bParametres.getSon()));
					serializer.endTag(null,"son");
					
				serializer.endTag(null,	"record");

			} catch (IllegalArgumentException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IllegalStateException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} catch (NullPointerException e) {
				LogAnniv.e (CLASSE, "NullPointerException:Certainement un bean avec un getter retournant un null:");
				e.printStackTrace();
			}
		}
	}
	
	
}
