package fr.free.supertos.anniversaire.database;

import fr.free.supertos.anniversaire.constantes.ConstantesAnniversaires;
import fr.free.supertos.anniversaire.log.LogAnniv;
import android.content.ContentValues;
import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteDatabase.CursorFactory;
import android.database.sqlite.SQLiteOpenHelper;

/**
 * Classe permettant de grer facilement l'accs  la BDD
 * 
 * @author supertos
 *
 */
public class AnnivSqLiteOpenHelper extends SQLiteOpenHelper {

	private final static String CLASSE = AnnivSqLiteOpenHelper.class.getName();
	
	public AnnivSqLiteOpenHelper(Context context, String name,
			CursorFactory factory, int version) {
		super(context, name, factory, version);
		if (LogAnniv.isDebug())
		{
			LogAnniv.d (CLASSE, "Initialisation MySQLiteOpenHelper:" + "Base=" + name  + ", Version=" + version);
		}
	}

	@Override
	/**
	 * Ouverture base
	 */
	public void onOpen (SQLiteDatabase db) {
		if (LogAnniv.isInformation())
		{
			LogAnniv.i (CLASSE, "Ouverture de la base.");
		}
	}
	
	/**
	 * Si la base n'existe pas, cration de celle-ci ainsi que des tables associes
	 */
	@Override
	public void onCreate(SQLiteDatabase db) {
		db.execSQL("create table anniversaires (id INTEGER PRIMARY KEY AUTOINCREMENT, idContact, nomprenom text NOT NULL, nom text, prenom text, annee INTEGER, mois INTEGER, jour INTEGER, anneeconnue INTEGER, autreeven INTEGER)");
		db.execSQL("CREATE INDEX index_nomprenom ON anniversaires (nomprenom)");
		db.execSQL("create table parametres (id INTEGER PRIMARY KEY AUTOINCREMENT, delaiRepeterAlarme integer,heureAlarme integer,minuteAlarme integer, vibration integer, son integer)");
		db.execSQL("create table etat (id INTEGER PRIMARY KEY AUTOINCREMENT, startams integer)");
		
		ContentValues cv = new ContentValues();
		// Cration de paramtres par dfaut
		cv.put ("delaiRepeterAlarme", "180");
		cv.put ("heureAlarme", "8");
		cv.put ("minuteAlarme", "30");
		cv.put ("vibration", "1");
		cv.put ("son", "1");
		db.insert("parametres", null, cv);
		
		// Cration enregistrement etat
		cv = new ContentValues();
		cv.put("startams", 0);
		db.insert("etat", null, cv);
		
		cv = new ContentValues();
		if (LogAnniv.isDebug())
		{
			LogAnniv.d (CLASSE, "Cration de la base.");
			
			// Debug - YYYY-MM-DD HH-MM-SS
 			// Et cration d'enregistrements de tests
			cv.put("nomprenom", "saisrien"+"jean");
 			cv.put("nom", "saisrien");
 			cv.put("prenom", "jean");
 			cv.put("annee", "1970");
 			cv.put("mois", "5");
 			cv.put("jour", 15);
 			cv.put("idContact", ConstantesAnniversaires.CONTACTINCONNUE.toString());
 			cv.put("anneeconnue", ConstantesAnniversaires.ANNEECONNUE);
 			cv.put("autreeven", ConstantesAnniversaires.EVENEMENTANNIVERSAIRE);
 			db.insert("anniversaires", null, cv);
 			cv.put("nomprenom", "mansoif"+"gerard");
 			cv.put("nom", "mansoif");
 			cv.put("prenom", "gerard");
 			cv.put("annee", "1980");
 			cv.put("mois", "6");
 			cv.put("jour", "9");
 			cv.put("anneeconnue", ConstantesAnniversaires.ANNEEINCONNUE);
 			cv.put("autreeven", ConstantesAnniversaires.EVENEMENTANNIVERSAIRE);
 			cv.put("idContact", ConstantesAnniversaires.CONTACTINCONNUE.toString());
 			db.insert("anniversaires", null, cv);
		}
	}

	/**
	 * Pas de gestion de la maj de la base dans cette version, car elle ne peut exister jusqu'ici !
	 */
	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
		// TODO Auto-generated method stub
		if (LogAnniv.isDebug())
		{
			LogAnniv.d (CLASSE, "Maj de la base.");
		}
	}
 	
}

