package fr.free.supertos.anniversaire.database.classes;

import java.io.ByteArrayInputStream;
import java.io.InputStream;

import android.content.ContentResolver;
import android.content.ContentUris;
import android.database.Cursor;
import android.net.Uri;
import android.provider.ContactsContract;
import android.provider.ContactsContract.Contacts;

/**
 * Recupraion d'informations sur un contact
 * photo, nom/prenom,...
 * @author supertos
 *
 */
public class ContactAnniv {
	private long id;

	private String nom;
	private String prenom;
	private String display;
	
	private ContentResolver contentResolver;
	
	public ContactAnniv(long id, ContentResolver contentResolver) {
		super();
		this.id = id;
		this.contentResolver=contentResolver;
		
		recupererInformationsContact(id);
	}

	public ContentResolver getContentResolver() {
		return contentResolver;
	}

	public long getId() {
		return id;
	}

	public InputStream openPhoto() {
	     Uri contactUri = ContentUris.withAppendedId(Contacts.CONTENT_URI, id);
	     Uri photoUri = Uri.withAppendedPath(contactUri, Contacts.Photo.CONTENT_DIRECTORY);
	     Cursor cursor = getContentResolver().query(photoUri, new String[] {Contacts.Photo.PHOTO}, null, null, null);
	     if (cursor == null) {
	         return null;
	     }
	     try {
	         if (cursor.moveToFirst()) {
	             byte[] data = cursor.getBlob(0);
	             if (data != null) {
	                 return new ByteArrayInputStream(data);
	             }
	         }
	     } finally {
	         cursor.close();
	     }
	     return null;
	}

	public void recupererInformationsContact(long id) {
		// recherche d'informations complmentaires, ici le nom et prenom du contact:
		String whereName = ContactsContract.Data.MIMETYPE + " = ? AND " + ContactsContract.CommonDataKinds.StructuredName.CONTACT_ID + " = ?";
		String[] whereNameParams = new String[] { ContactsContract.CommonDataKinds.StructuredName.CONTENT_ITEM_TYPE, String.valueOf(id) };
		Cursor nameCur = getContentResolver().query(ContactsContract.Data.CONTENT_URI, null, whereName, whereNameParams, ContactsContract.CommonDataKinds.StructuredName.GIVEN_NAME);
		if (nameCur != null && nameCur.moveToFirst()) {
		     prenom = nameCur.getString(nameCur.getColumnIndex(ContactsContract.CommonDataKinds.StructuredName.GIVEN_NAME));
		     nom = nameCur.getString(nameCur.getColumnIndex(ContactsContract.CommonDataKinds.StructuredName.FAMILY_NAME));
		     display = nameCur.getString(nameCur.getColumnIndex(ContactsContract.CommonDataKinds.StructuredName.DISPLAY_NAME));
	    }
		else
		{
			prenom=null;
			nom=null;
			display=null;
		}
	    nameCur.close();
	}	  

	public String getNom() {
		return nom;
	}
	
	public String getPrenom() {
		return prenom;
	}
	
	public String getDisplay() {
		return display;
	}

}
