package fr.free.supertos.anniversaire.service;

import java.util.Calendar;

import android.app.AlarmManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.os.IBinder;
import fr.free.supertos.anniversaire.bean.BeanEtat;
import fr.free.supertos.anniversaire.bean.BeanParametres;
import fr.free.supertos.anniversaire.dao.DaoEtat;
import fr.free.supertos.anniversaire.dao.DaoParametres;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;

/**
 * Service auto run
 * Ce service est lanc automatiquement lorsque le systme a fini de booter
 * 
 * Son rle tant de surveiller priodiquement s'il y a ou non des anniversaires  souhaiter
 * 
 * @author supertos
 *
 */
public class ServiceAutoRun extends Service {
	
	private final static String CLASSE = ServiceAutoRun.class.getName();
	protected SQLiteDatabase idSqLite;
	protected SqLiteAnniversaire sqLiteAnniv;
	private boolean servicePremierAppel;
		
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Cration du service
	 * Recherche d'un identifiant sur la base de donnes pour ouverture et lancement d'un thread SurveillanceAnniversaireRunnable
	 * 
	 */
	@Override
	public void onCreate ()
	{

		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"Cration du service + lancement d'un thread.");
    	}	
	}
	
	/**
	 * Fin du service, arrt du thread SurveillanceAnniversaireRunnable associ, clture de la base de donnes
	 */
	@Override
	public void onDestroy() {
		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"Destruction du service");
    	}		
	}
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		
		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"Startcommand :" + intent.getBooleanExtra("CHANGE",false));
    	}	
		
		if (intent.getBooleanExtra("CHANGE",false))
		{
			// On simule un premier dmarrage
			servicePremierAppel=true;
		}
			
		sqLiteAnniv = new SqLiteAnniversaire();
		idSqLite=sqLiteAnniv.openDB(this);

		DaoParametres daoParametres=(DaoParametres) DaoFactory.daoParametres(idSqLite);
		BeanParametres bParametres=daoParametres.getParametres();

		DaoEtat daoEtat=(DaoEtat) DaoFactory.daoEtat(idSqLite);
		BeanEtat bEtat=daoEtat.getEtat();

		Calendar calendar = Calendar.getInstance();
		long nowMilli= System.currentTimeMillis();
				
		calendar.set(calendar.get(Calendar.YEAR), calendar.get(Calendar.MONTH), calendar.get(Calendar.DAY_OF_MONTH), bParametres.getHeureAlarme(), bParametres.getMinuteAlarme());

		if (bEtat.getStartAMS()/1000>=calendar.getTimeInMillis()/1000) // Pas besoin de precision au millionieme, sinon on va rentrer en boucle pendant 1 sec...
		{
			// Le service a dj t lanc aujourd'hui pour traiter l'alarme. Donc rien  faire
			LogAnniv.d (CLASSE,"Startcommand : Appel deja fait - " + calendar.getTimeInMillis() );
			servicePremierAppel=false;
		}
		else
		{
			// Premier lancement du service et une alarme manque, ca commence bien !
			LogAnniv.d (CLASSE,"Startcommand : Premier Appel !" + calendar.getTimeInMillis() );
			servicePremierAppel=true;
		}
		
		bEtat.setStartAMS(nowMilli);
		// On sauve le fait que nous avons trait ce jour la programmation des alarmes
		// Ainsi, en cas de relance du portable, l'application ne va pas resignaler quelque chose qui l'a deja ete.
		// Hormis si reprogrammation alarme, dans ce cas, tout dpendra de la valeur de celle-ci, si reprogramme aprs la premiere, cela sera signale...
		daoEtat.ecrireEtat(bEtat);
		
		Intent intentService=new Intent (this, ServiceCtrlAnniversaire.class);
		if (bParametres.getDelaiRepeterAlarme()==0)
		{
			intentService.putExtra("Relance", true);
		}
		else
		{
			// Rptition demande, elle ne cessera que lorsque le cancel sur ce service arrivera, donc au lancement de notre application dans le MainActivity qui fait appel  ce service.
			intentService.putExtra("Relance", false);
		}
		
		PendingIntent pendingIntent = PendingIntent.getService(this, 0, intentService, PendingIntent.FLAG_UPDATE_CURRENT);
		
		AlarmManager alarms = (AlarmManager)getSystemService(Context.ALARM_SERVICE);
		// Suppression alarme prcdente (et surtout de sa rptition !)
		alarms.cancel(pendingIntent);

		if (servicePremierAppel)
		{
			servicePremierAppel=false;
			if (LogAnniv.isDebug())
	    	{
				LogAnniv.d (CLASSE,"Premier appel du service ! Programmation alarme  " + calendar.get(Calendar.YEAR) + "/"+ (calendar.get(Calendar.MONTH) + 1) + "/" + calendar.get(Calendar.DAY_OF_MONTH) + " " + bParametres.getHeureAlarme() + ":" + bParametres.getMinuteAlarme() + "=>" + calendar.getTimeInMillis() + " Maintenant:" + nowMilli);
				LogAnniv.d (CLASSE,"Premier appel du service ! Programmation Rappel  " + bParametres.getDelaiRepeterAlarme()*60000);
	    	}	
			
			if (bParametres.getDelaiRepeterAlarme()==0) // Alarme sans rptition
			{
				alarms.set(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), pendingIntent); // Si l'alarme est passe, elle sera dclenche immdiatement !
			}
			else
			{
				// Rptition demande, elle ne cessera que lorsque le cancel sur ce service arrivera, donc au lancement de notre application dans le MainActivity qui fait appel  ce service.
				alarms.setRepeating(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), bParametres.getDelaiRepeterAlarme()*60000, pendingIntent);

			}
		}
		else
		{
			calendar.setTimeInMillis(calendar.getTimeInMillis() + AlarmManager.INTERVAL_DAY);

			if (LogAnniv.isDebug())
	    	{
				LogAnniv.d (CLASSE,"nieme appel du service ! Programmation alarme  " + calendar.get(Calendar.YEAR) + "/"+ (calendar.get(Calendar.MONTH) + 1) + "/" + calendar.get(Calendar.DAY_OF_MONTH) + " " + bParametres.getHeureAlarme() + ":" + bParametres.getMinuteAlarme() + "=>" + calendar.getTimeInMillis() + " Maintenant:" + nowMilli);
				LogAnniv.d (CLASSE,"nieme appel du service ! Programmation Rappel  " + bParametres.getDelaiRepeterAlarme()*60000);
	    	}	
			// Programmation d'une alarme  l'heure indiqu + 24h
			if (bParametres.getDelaiRepeterAlarme()==0) // Alarme sans rptition
			{
				alarms.set(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), pendingIntent);

			}
			else
			{
				alarms.setRepeating(AlarmManager.RTC_WAKEUP, calendar.getTimeInMillis(), bParametres.getDelaiRepeterAlarme()*60000, pendingIntent);
			}
		}
		
		sqLiteAnniv.closeDB (idSqLite);		

		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"FIN Startcommand !");
    	}	

		stopSelf(startId);
		
		return START_NOT_STICKY;
	}   
}
