package fr.free.supertos.anniversaire.service;

import java.util.Calendar;
import java.util.List;

import android.app.Notification;
import android.app.NotificationManager;
import android.app.PendingIntent;
import android.app.Service;
import android.content.Context;
import android.content.Intent;
import android.database.sqlite.SQLiteDatabase;
import android.net.Uri;
import android.os.Build;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import fr.free.supertos.anniversaire.MainActivity;
import fr.free.supertos.anniversaire.R;
import fr.free.supertos.anniversaire.bean.BeanAnniversaires;
import fr.free.supertos.anniversaire.bean.BeanParametres;
import fr.free.supertos.anniversaire.dao.DaoAnniversaires;
import fr.free.supertos.anniversaire.dao.DaoParametres;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;

/**
 * Service auto run
 * Ce service est lanc automatiquement lorsque le systme a fini de booter
 * 
 * Son rle tant de surveiller priodiquement s'il y a ou non des anniversaires  souhaiter
 * 
 * @author supertos
 *
 */
public class ServiceCtrlAnniversaire extends Service {
	
	private final static String CLASSE = ServiceCtrlAnniversaire.class.getName();
	private static Thread monThread=null;
	protected SQLiteDatabase idSqLite;
	protected SqLiteAnniversaire sqLiteAnniv;
	
	@Override
	public IBinder onBind(Intent arg0) {
		// TODO Auto-generated method stub
		return null;
	}

	/**
	 * Cration du service
	 * Recherche d'un identifiant sur la base de donnes pour ouverture et lancement d'un thread SurveillanceAnniversaireRunnable
	 * 
	 */
	@Override
	public void onCreate ()
	{

		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"Cration du service Ctrl anniversaires.");
    	}
	
		super.onCreate();
	}
	
	/**
	 * Fin du service, arrt du thread SurveillanceAnniversaireRunnable associ, clture de la base de donnes
	 */
	@Override
	public void onDestroy() {
		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"Destruction du service");
    	}
		
		if (monThread!=null)
		{
			monThread.interrupt();
		}
		
		super.onDestroy();		
	}
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"*** Startcommand !");
    	}	

		// relance permet de savoir si vrai, que le service sera relanc car l'alarme est programm
		// avec une rptition. Si  faux, il ne sera pas relanc. Il faudra donc relancer le service ServiceAutoRun 
		// afin que celui-ci reprogramme une alarme...
		boolean relance=intent.getBooleanExtra("Relance", false);
		CtrlAnniversaireRunnable ctrlAnniversaire = new CtrlAnniversaireRunnable(handlerMsg, this, relance);
		monThread = new Thread(ctrlAnniversaire);
		monThread.start();

		if (LogAnniv.isDebug())
    	{
			LogAnniv.d (CLASSE,"*** FIN Startcommand !");
    	}	

		stopSelf(startId);
		
	    return START_NOT_STICKY;
	}

	/**
	 * Classe handler pour permettre au thread SurveillanceAnniversaireRunnable de communiquer avec le service (THREAD UI)
	 */
	Handler handlerMsg= new Handler()
    {
		public void handleMessage(android.os.Message msg)
    	{
  		
    		switch (msg.what)
    		{			
				
				case 1:
				{
					// debug
					if (LogAnniv.isDebug())
					{
						LogAnniv.d (CLASSE,"Surveillance trt en cours...Il y a des anniversaires");
					}
					
					 int icon = R.drawable.ic_launcher;

					 String tickerText = getResources().getString(R.string.ticker);
					 
					 long timeMilli = System.currentTimeMillis();


					 Context context = ServiceCtrlAnniversaire.this.getApplicationContext();
					 
					 Intent monIntent = new Intent(ServiceCtrlAnniversaire.this, MainActivity.class);
					 PendingIntent pendingIntent = PendingIntent.getActivity(context, 0, monIntent, Intent.FLAG_ACTIVITY_NEW_TASK);
					 
					 Notification notification;
					 
					 if (Build.VERSION.SDK_INT < Build.VERSION_CODES.HONEYCOMB)
					 {
						 if (LogAnniv.isDebug())
 						 {
							LogAnniv.d (CLASSE,"API < 11 (HONEYCOMB)");
						 }
						 notification = new Notification();
	
						 notification.icon=icon;
						 notification.tickerText=tickerText;
						 notification.when=timeMilli;
						 notification.setLatestEventInfo(context, getText(R.string.app_name), tickerText, pendingIntent);
					 }
				     else
				     {
						 if (LogAnniv.isDebug())
 						 {
							LogAnniv.d (CLASSE,"API < 11 (HONEYCOMB)");
						 }
	
					     notification = new Notification.Builder(context)
				          .setContentTitle(getText(R.string.app_name))
				          .setContentText(tickerText)
				          .setTicker(tickerText)
				          .setWhen(timeMilli)
				          .setContentIntent(pendingIntent)
				          .setAutoCancel(true)
				          .setSmallIcon(icon)
				          .build();
				     }
					
					 NotificationManager notificationManager = (NotificationManager)getSystemService(Context.NOTIFICATION_SERVICE);
					 
				     // Vibration
				     if (msg.arg1==1)
				    	 notification.vibrate = new long[] { 100, 250, 100, 500};

				     // son
				     if (msg.arg2==1)
				    	 notification.sound = Uri.parse("android.resource://" + getPackageName() + "/" + R.raw.music_banjo_happy_birthday_nifter_dot_com);
				     
				     // Envoyer la notification
				     notificationManager.notify(1, notification);				     
			        break;
				}

				case 2:
				{
				     // Relance du service pour reprogrammer une nouvelle alarme
				     // Ncessaire dans le cas d'une alarme sans rptition, afin de la reprogrammer pour dans 24h 

					if (LogAnniv.isDebug())
			    	{
						LogAnniv.d (CLASSE,"*** Relance demande !");
			    	}	
			    	Intent intentServiceAutoRun = new Intent(ServiceCtrlAnniversaire.this, ServiceAutoRun.class);	    		
			    	startService(intentServiceAutoRun);
			        break;
				}
    		}
		    removeMessages(msg.what); // Supprimer le message qui ne servira plus maintenant que nous l'avons trait.
    	}
    };
    
	/**
	 * Thread lanc par le service lors de sa creation afin de vrifier priodiquement les anniversaires
	 */
	public class CtrlAnniversaireRunnable implements Runnable {
		private Handler handlerMsg;
		private SQLiteDatabase idSqLite;
		private Context context;
		private boolean flagRelance;
		
		/**
		 * Constructeur qui doit recevoir un Handler permettant au thread de communiquer avec le service, et l'identifiant de la base de donnes
		 * afin d'accder aux tables.
		 * @param handlerMsg
		 * @param flags 
		 * @param idSqLite
		 */
		public CtrlAnniversaireRunnable(Handler handlerMsg, Context context, boolean relance) {
			this.handlerMsg=handlerMsg;
			this.context=context;
			this.flagRelance=relance;
			
		}


		/**
		 * Mthode run contenant le code  excuter pour le thread
		 * Le thread ne se termine que si un interrupt() est lanc dessus
		 */
		@Override
		public void run() {
			Message msg;
			Calendar date;
			
			if (LogAnniv.isDebug())
	    	{
				LogAnniv.d (CLASSE,"*** Thread...");
	    	}	

			sqLiteAnniv = new SqLiteAnniversaire();
			idSqLite=sqLiteAnniv.openDB(context);

			DaoAnniversaires daoAnniversaires=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
			
			date = Calendar.getInstance();
			
			List<BeanAnniversaires> listAnniversaires=daoAnniversaires.listAnniversaires(date.get(Calendar.YEAR),date.get(Calendar.MONTH) + 1,date.get(Calendar.DAY_OF_MONTH));
			
			if (!listAnniversaires.isEmpty())
			{
				DaoParametres daoParametres=(DaoParametres)DaoFactory.daoParametres(idSqLite);
				BeanParametres bParamtres = daoParametres.getParametres();
				// Il y a des anniversaires aujourd'hui !
				msg=handlerMsg.obtainMessage(1, bParamtres.getVibration(), bParamtres.getSon());
				handlerMsg.sendMessage(msg);
			}

			if (flagRelance)
			{
				msg=handlerMsg.obtainMessage(2);
				handlerMsg.sendMessage(msg);
			}
			sqLiteAnniv.closeDB (idSqLite);
			
			if (LogAnniv.isDebug())
	    	{
				LogAnniv.d (CLASSE,"*** FIN Thread...");
	    	}	

		}
	}	
}
