package fr.free.supertos.anniversaire.service;

import java.util.Calendar;
import java.util.List;

import android.app.Service;
import android.appwidget.AppWidgetManager;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.res.Resources;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.os.IBinder;
import android.widget.RemoteViews;
import fr.free.supertos.anniversaire.AnniversaireWidget;
import fr.free.supertos.anniversaire.R;
import fr.free.supertos.anniversaire.bean.BeanAnniversaires;
import fr.free.supertos.anniversaire.dao.DaoAnniversaires;
import fr.free.supertos.anniversaire.database.SqLiteAnniversaire;
import fr.free.supertos.anniversaire.factory.DaoFactory;
import fr.free.supertos.anniversaire.log.LogAnniv;

public class ServiceUpdateAppWidget extends Service {
	
	private RemoteViews views;
	private ThreadTraitementLong maTache;
	private ComponentName componentNameSurAppWidget;
	private AppWidgetManager appWidgetManager;
	
	private SqLiteAnniversaire sqLiteAnniv=null;
	private SQLiteDatabase idSqLite=null;
	
	// Les enregistrements qui alimenteront la liste des lments de la liste
	private List<BeanAnniversaires> listAnniversaires;
	private List<BeanAnniversaires> listFutursAnniversaires;
	
	private Context baseContext;
	
	private final static String CLASSE = ServiceUpdateAppWidget.class.getName();
	
	@Override
	public int onStartCommand(Intent intent, int flags, int startId) {
        LogAnniv.d(CLASSE, "onStartCommand()");

        // Rcupration d'une instance AppWidgetManager pour notre contexte...
        appWidgetManager = AppWidgetManager.getInstance(this);

        // Rcupration d'une instance ComponentName sur le package et la classe AnniversaireWidget  
        componentNameSurAppWidget = new ComponentName(this, AnniversaireWidget.class);
        // Cette instance sera fournie plus tard  l'appWidgetManager pour maj de toutes les instances de notre appWidget de la classe indique
        // dans ComponentName
        
        // Lancement du traitement de maj du contenu des appwidgets
        baseContext=this;
        
        buildUpdate(baseContext);
        LogAnniv.d(CLASSE, "update built");
        
	    return START_NOT_STICKY;

    }

    @Override
    public IBinder onBind(Intent intent) {
        return null;
    }
   
    public void buildUpdate(Context context) {
        // Construction des maj  apporter dans l'appWidget
    	views = new RemoteViews(context.getPackageName(), R.layout.widget_layout);
        
    	views.setTextViewText(R.id.anniversaire_info, "Traitement en cours...");

    	// Mettre  jour les vues des instances de notre AppWidget
    	appWidgetManager.updateAppWidget(componentNameSurAppWidget, views);
    	                    
    	maTache=new ThreadTraitementLong();
		maTache.execute();		
    }
    
    class ThreadTraitementLong extends AsyncTask<Integer, Long, Void> {

		@Override
		protected void onPreExecute() {
			views.setTextViewText(R.id.anniversaire_info, getString(R.string.init_app_widget));
	        views.setTextViewText(R.id.anniversaire_info_even, "");
	        views.setTextViewText(R.id.anniversaire_info_even_bis, "");
	        views.setTextViewText(R.id.anniversaire_info_even_ter, "");
	        views.setTextViewText(R.id.anniversaire_info_even_quater, "");
	        views.setTextViewText(R.id.anniversaire_info_even_quinquies, getString(R.string.wait));
	        appWidgetManager.updateAppWidget(componentNameSurAppWidget, views);
		}

		@Override
		protected void onProgressUpdate(Long... values){
			// Rien  faire...
		}

		@Override
		protected Void doInBackground(Integer... arg0) {

			LogAnniv.d(CLASSE, "Avant Sleep");

			sqLiteAnniv = new SqLiteAnniversaire();
			idSqLite=sqLiteAnniv.openDB(baseContext);
			
			DaoAnniversaires daoAnniversaires=(DaoAnniversaires)DaoFactory.daoAnniversaires(idSqLite);
			
			Calendar c = Calendar.getInstance(); // date du jour
			listAnniversaires=daoAnniversaires.listAnniversaires(c.get(Calendar.YEAR), c.get(Calendar.MONTH)+1, c.get(Calendar.DAY_OF_MONTH));
			
			if (listAnniversaires.isEmpty())
			{
				// Recherche des prochains anniversaires
				listFutursAnniversaires=daoAnniversaires.listFutursAnniversaires();
			}
			// Fermeture accs base
			if (sqLiteAnniv!=null && idSqLite!=null)
				sqLiteAnniv.closeDB (idSqLite);
				
			return null;				
		}

		@Override
		protected void onPostExecute(Void result) {
			
			if (!listAnniversaires.isEmpty())
			{
				// Un ou plusieurs anniversaires  fter !
				
				// Gestion du pluriel pour anniversaire
				Resources resources = getResources();
				
				String strAnniversaire = resources.getQuantityString(R.plurals.texteAnniversaire, listAnniversaires.size(), (Object[]) null);

				views.setTextViewText(R.id.anniversaire_info, strAnniversaire);

				if (listAnniversaires.size()==1)
				{
					
					views.setTextViewText(R.id.anniversaire_info_even_bis, listAnniversaires.get(0).getNom() + " " + listAnniversaires.get(0).getPrenom());
					views.setTextViewText(R.id.anniversaire_info_even_quinquies, "");
				}
				else
				{
					views.setTextViewText(R.id.anniversaire_info_even_bis, listAnniversaires.get(0).getNom() + " " + listAnniversaires.get(0).getPrenom());
					if (listAnniversaires.size()>=2)
					{
						views.setTextViewText(R.id.anniversaire_info_even_ter, listAnniversaires.get(1).getNom() + " " + listAnniversaires.get(1).getPrenom());
					}
					
					if (listAnniversaires.size()>=3)
					{
						views.setTextViewText(R.id.anniversaire_info_even_quater, listAnniversaires.get(2).getNom() + " " + listAnniversaires.get(2).getPrenom());
					}

					if (listAnniversaires.size()>=4)
					{
						views.setTextViewText(R.id.anniversaire_info_even_quinquies, getString (R.string.etc));
					}

				}
				
				
			}
			else
			{
				// Le prochain anniversaire sera dans n jours...
				views.setTextViewText(R.id.anniversaire_info, getString(R.string.prochainAnniv));
				
				views.setTextViewText(R.id.anniversaire_info_even, "");
				views.setTextViewText(R.id.anniversaire_info_even_bis, nbJours(listFutursAnniversaires.get(0).getNbJoursAvantAnniversaire()) + ":" + listFutursAnniversaires.get(0).getNom() + " " + listFutursAnniversaires.get(0).getPrenom());
				
				if (listFutursAnniversaires.size()>=2)
				{
					views.setTextViewText(R.id.anniversaire_info_even_ter, nbJours(listFutursAnniversaires.get(1).getNbJoursAvantAnniversaire()) + ":" + listFutursAnniversaires.get(1).getNom() + " " + listFutursAnniversaires.get(1).getPrenom());
				}
				
				if (listFutursAnniversaires.size()>=3)
				{
					views.setTextViewText(R.id.anniversaire_info_even_quater, nbJours(listFutursAnniversaires.get(2).getNbJoursAvantAnniversaire()) + ":" + listFutursAnniversaires.get(2).getNom() + " " + listFutursAnniversaires.get(2).getPrenom());
				}

				if (listFutursAnniversaires.size()>=4)
				{
					views.setTextViewText(R.id.anniversaire_info_even_quinquies, getString (R.string.etc));
				}
				else
				{
					views.setTextViewText(R.id.anniversaire_info_even_quinquies, "");
				}

			}
			LogAnniv.d("SERVICE", "Demande maj Views !");
	        appWidgetManager.updateAppWidget(componentNameSurAppWidget, views);
	        stopSelf(); // Fin du service !
		}

		private String nbJours(Integer nbJoursAvantAnniversaire) {
			Resources res = getResources();
			String jours=res.getQuantityString(R.plurals.texteJour, nbJoursAvantAnniversaire, nbJoursAvantAnniversaire);
			return jours;
		}
	}
}