package fr.free.supertos.anniversaire.xml;

import java.io.File;
import java.io.IOError;

import android.content.Context;
import android.os.Build;
import android.os.Environment;
import fr.free.supertos.anniversaire.classes.FiltreNom;
import fr.free.supertos.anniversaire.log.LogAnniv;
import fr.free.supertos.anniversaire.zip.ZipAnniversaire;

/**
 * Cette classe permet de grer les fichiers xml qui sont zipps (extraction/compression)
 * 
 * Une contrainte cependant, il faut que le priphrique possde une carte SD, ce qui n'est pas le cas de tous les appareils...
 * 
 * @author Internet
 *
 */
public class XmlAnniversaire {

	private final static String CLASSE = XmlAnniversaire.class.getName();

	private File repertoireAnniversaire;
	
	private boolean openZip;
	
	/**
	 * Importation des donnes.
	 * @param contexte
	 * @return
	 */
	public File openXML (Context contexte) {

		if (LogAnniv.isInformation())
		{
			LogAnniv.i (CLASSE, "openXML...");
		}
		openXMLFile(contexte);
		
		openZip=true;
		return repertoireAnniversaire;
    }
	
	/**
	 * une exportation des donnes, il faut donc craser le prcdent zip
	 * @return
	 */
	public File createXMLFile(Context contexte) {
		// Cela signifie qu'il faut supprimer le prcdent fichier zip...
		repertoireAnniversaire = detecterCarteSD(contexte);
		
		// Vrifier la prsence d'un fichier zip
    	File fichierZip=new File (repertoireAnniversaire.getAbsolutePath() + "/anniversaire.zip");
    	if (fichierZip.exists() && fichierZip.isFile())
    	{
    		fichierZip.delete();
    	}
    	openZip=true;
    	return repertoireAnniversaire;
	}
	
	/**
	 * Dtermination de la prsence de la carte SD, et prparation du chemin qui sera utilis
	 * pour le fichier zip et les xml crs
	 * @return
	 */
	public File detecterCarteSD(Context contexte) {
		// Recherche du premier support de stockage externe (cela peut tre la carte SD mais aussi une mulation)
		String state = Environment.getExternalStorageState();
		String fic=null;
		if (Environment.MEDIA_MOUNTED.equals(state))
        {
			File sdCard=null;
			
			// OK, la carte est monte, rcupration du chemin de montage (Unix oblige)
			// Si API >=19 et le support est SD (donc non mul), nous n'avons normalement pas accs n'importe 
			// o  la carte SD. Seul le rpertoire package est disponible...
			if (Environment.isExternalStorageRemovable () && Build.VERSION.SDK_INT>=19) //Build.VERSION_CODES.KITKAT
			{
            	sdCard=new File (contexte.getExternalFilesDir (Environment.getDataDirectory().getAbsolutePath()).getAbsolutePath());
            	fic = sdCard.getAbsolutePath();
			}
			else // Sinon, pas de pb !
			{
            	sdCard=Environment.getExternalStorageDirectory();
            	fic = sdCard.getAbsolutePath()+ "/anniversaire";
			}
            
            if (LogAnniv.isDebug())
            {
            	LogAnniv.d ("Test SDCARD:", sdCard.getAbsolutePath());
            }
	        
	        
	        // Controler la prsence du rpertoire anniversaire
	        File repertoireAnniversaire = new File (fic);
	        
	        if (!repertoireAnniversaire.exists() || !repertoireAnniversaire.isDirectory())
	        {
	        	// Absent, donc le crer...
	        	if (!repertoireAnniversaire.mkdir())
	        	{
	        		//throw  new IOError (new Throwable ("Le rpertoire " + fic + " existe dj ou est impossible  crer."));
	        		// On retourne le rpertoire courant
	        		return contexte.getFilesDir();
	        	}
	        	else
	        	{
	        		return repertoireAnniversaire;
	        	}
	        }
	        else
	        {
	        	// Ctrl la possibilit de lire/crire dedans
	        	if (repertoireAnniversaire.canRead() && repertoireAnniversaire.canWrite())
	        	{
	        		return repertoireAnniversaire;
	        	}
	        	else
	        	{
	        		return contexte.getFilesDir();
	        		// throw  new IOError (new Throwable ("Le rpertoire " + fic + " n'est pas accessible en lecture ou criture."));
	        	}
	        }	        
        }
		else
		{
			// throw  new IOError (new Throwable ("La carte SD n'est pas monte"));
			return contexte.getFilesDir();
		}
	}

	/**
	 * A priori, on demande une importation des donnes, il faut donc ouvrir le zip et extraire les fichiers xml pour une future lecture
	 * des donnes
	 */
	public void openXMLFile(Context contexte) {
		
		repertoireAnniversaire = detecterCarteSD(contexte);

		if (repertoireAnniversaire != null)
		{		        	
        	// Vrifier la prsence d'un fichier zip
			// Ici, le fichier zip et les .xml seront dans le mme repertoire au final
        	File fichierZip=new File (repertoireAnniversaire.getAbsolutePath() + "/anniversaire.zip");
        	if (fichierZip.exists() && fichierZip.isFile())
        	{
        		if (fichierZip.canRead() && fichierZip.canWrite())
        		{
        			ZipAnniversaire zip = new ZipAnniversaire();
        			zip.unZip(fichierZip, repertoireAnniversaire);
        		}
        		else
        		{
        			throw  new IOError (new Throwable ("Le fichier zip " + repertoireAnniversaire.getAbsolutePath() + "/anniversaire.zip n'est pas accessible en lecture ou criture."));
        		}
        	}
        }
	}

	/**
	 * Fermeture, cela se traduit par zipper tous les fichiers xml puis suppression des xml.
	 * @param idSQLiteDB
	 */
	public void closeXMLFile ()
	{	
		// Rien  fermer si rien d'ouvert !
		if (LogAnniv.isInformation())
        {
			LogAnniv.i (CLASSE, "Fermeture XML.");
        }

		if (openZip)
		{
			
			// Zippage des fichiers XML
			// On commence donc par supprimer le zip prcdent
			File ancZip=new File (repertoireAnniversaire+"/anniversaire.zip");
			ancZip.delete();
			
			// Puis zip ...
			ZipAnniversaire zip = new ZipAnniversaire();
			File fichierZip=new File (repertoireAnniversaire.getAbsolutePath() + "/anniversaire.zip");
			
			// Note, le fichier zip et les .xml sont ici dans le mme rpertoire
			zip.zip (fichierZip, repertoireAnniversaire);
			
			// On peut supprimer tous les fichiers XML maintenant
			if (repertoireAnniversaire!=null)
			{
				FiltreNom instFiltreNom = new FiltreNom();
				String[] fichier = repertoireAnniversaire.list(instFiltreNom);
				for (String element : fichier)
				{
					File xml = new File (repertoireAnniversaire+"/"+element);
					 if (LogAnniv.isDebug())
			         {
			           	LogAnniv.d ("closeXMLFile:", " Delete " + xml.getAbsolutePath());
			         }
					xml.delete();
				}
			}			
		}
	}
}
