	package fr.free.supertos.anniversaire.zip;

	import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.zip.Deflater;
import java.util.zip.ZipEntry;
import java.util.zip.ZipInputStream;
import java.util.zip.ZipOutputStream;

import fr.free.supertos.anniversaire.classes.FiltreNom;
import fr.free.supertos.anniversaire.log.LogAnniv;

/**
 * Cette classe permet de gnrer un zip anniversaire.zip compos de tous les fichiers .xml qui ont pu tre
 * gnr dans le rpertoire utilis
 * ou  l'inverse, d'extraire dans le rpertoire utilis tous les .xml du fichier zip 
 * @author supertos
 *
 */
public class ZipAnniversaire {

	static final int BUFFER = 2048;
	
	/**
	 * zip des fichiers xml
	 * @param fichierZip Nom du fichier zip  produire
	 * @param repertoireAnniversaire Rpetoire contenant les fichiers xml
	 */
	public void zip (File fichierZip, File repertoireAnniversaire)
	{
		
		FileOutputStream destZip;
		try {
			destZip = new FileOutputStream (fichierZip);
			
			BufferedOutputStream bufferOuput = new BufferedOutputStream (destZip);
			
			// Prparation au zip...
			ZipOutputStream outZip = new ZipOutputStream (bufferOuput);
			
			outZip.setMethod (ZipOutputStream.DEFLATED);
			outZip.setLevel(Deflater.BEST_COMPRESSION);

			// Traitement des fichiers xml
			// Recherche des fichiers XML prsents dans le rpertoire...
			FiltreNom instFiltreNom = new FiltreNom();
			String[] fichier = repertoireAnniversaire.list(instFiltreNom);
			
			for (String element : fichier)
			{
				File xml = new File (repertoireAnniversaire+"/"+element);
				FileInputStream sourceXML;
				try {
					sourceXML = new  FileInputStream(xml);
					
					 if (LogAnniv.isDebug())
			         {
			           	LogAnniv.d ("zip", "Traitement fichier :" + xml.getAbsolutePath());
			         }
			

					BufferedInputStream bufferInput = new BufferedInputStream(sourceXML);

					try {
						ZipEntry zipEntry = new ZipEntry(element);
						outZip.putNextEntry(zipEntry);
						
						// Compression d'un fichier xml
						int carac;
						do
						{
							carac=bufferInput.read();
							
							if (carac != -1)
							{
								outZip.write (carac);
							}
						} while (carac != -1);
						
						
						outZip.closeEntry();
			
							
						
					} catch (FileNotFoundException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}

					try {
						bufferInput.close();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					
				} catch (FileNotFoundException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			}
			outZip.close();
			bufferOuput.close();
			
			destZip.close();

		} catch (FileNotFoundException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		
	}

	
	/**
	 * Extraction des fichiers du fichier zip
	 * @param fichierZip Nom du fichier zip
	 * @param repertoireAnniversaire Nom du repertoire qui recevra les .xml
	 */
	public void unZip (File fichierZip, File repertoireAnniversaire) 
	{

		byte data[] = new byte[BUFFER];
		
		// Flux zip
		FileInputStream inZip;
		try {
			inZip = new FileInputStream(fichierZip);
			
			// et filtre buffer et zip pour ce flux
			BufferedInputStream bufferInput = new BufferedInputStream(inZip);
			ZipInputStream zipInput = new ZipInputStream(bufferInput);

			// Traitement des entres du zip
			ZipEntry entry;
			try {
				while((entry = zipInput.getNextEntry()) != null) {

					//Cration du fichier de sortie  partir du nom de lentre
					FileOutputStream fileOutput;
					try {
						fileOutput = new FileOutputStream(repertoireAnniversaire.getAbsoluteFile() + "/" + entry.getName());
						BufferedOutputStream bufferOutput = new BufferedOutputStream(fileOutput, BUFFER);

						int taille;
						// crire les donnes dzippes dans le fichier de sortie
						try {
							while ((taille = zipInput.read(data, 0, BUFFER)) != -1) {
								bufferOutput.write(data, 0, taille);
							}
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
				
						// Vidage du cache en criture
						try {
							bufferOutput.flush();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
				
						// Fermeture du fichier en sortie
						try {
							bufferOutput.close();
							fileOutput.close();
						} catch (IOException e) {
							// TODO Auto-generated catch block
							e.printStackTrace();
						}
					} catch (FileNotFoundException e1) {
						// TODO Auto-generated catch block
						e1.printStackTrace();
					}
				}
			} catch (IOException e1) {
				// TODO Auto-generated catch block
				e1.printStackTrace();
			}

			// Et zou termin...
			try {
				zipInput.close();
				bufferInput.close(); // Le filtre buffer
				inZip.close(); // Et le flux		
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			} // le filtre Zip
		} catch (FileNotFoundException e2) {
			// TODO Auto-generated catch block
			e2.printStackTrace();
		}	
	}
}
